--[[
	CombatStats - Stat Damage Tracking.  A World of Warcraft(tm) addOn to track basic combat stats.
	ORIGINAL AUTHOR: DmgInc on most forums / Glacier on official WoW forums
	WoW 2.0 ON AUTHOR: Brian Beardmore - wowaddon(at)realisticsoftware(dot)com

	Revisions:
		2011.07.03	BAB	version 4.901
			- Updated for WoW 4.2 interfaces
		05.05.2011	BAB	fixed the options drop down menu going inactive at first use.
		05.01.2011	BAB version 4.801
			- Updated for WoW 4.1 interfaces
		12.17.2010	BAB	version 4.702
			- Updated the frameStrata to a lower level  
		11.04.2010	BAB	version 4.701
			- Fixed dropdown menu problems 
		10.13.2010	BAB	version 4.700
			- Updated to get it working again for WoW 4.0 interfaces
			- Need improvements to better support the 4.0 overall changes.
		02.21.2010	BAB	version 4.682
			- fixed an initial nil value for gCS.dataVersion
		02.19.2010	BAB	version 4.681
			- Removed old combatstats stored data conversion functionality.  Pre version 4.64
			  data is now reset upon the addon loading.
		01.22.2010
			- Fixed the blizzard 3.3 menu problem
			   [see http://forums.wowace.com/showthread.php?t=15763]
			- Fixed heals to not all be crits
		12.11.2009	BAB	version 4.680
			- Updated for WoW 3.3 interfaces
			- Global usage code cleanup
			- Remove XML infobar information and create it dynamically if its needed
			- Fixed lua log errors
		08.04.2009	BAB version 4.670
			- Updated for WoW 3.2 interfaces
		04.16.2009	BAB version 4.660
			- Updated for WoW 3.1 interfaces
			- cleanup old 3.0 and 2.4 special switch code
		10.19.2008	BAB version 4.651
			- Fixed variable type usage in chatCommandHandler
		10.12.2008	BAB version 4.650
			- Updated for WoW 3.0 interfaces
			- More combat log changes in the new interfaces
			- CombatStats_Config flags have been changed to be true or false/nil from 0 and 1
			  this is more in line with blizzard flagging and other addons
			- Both 2.43 and 3.00 are supported in this release
		09.14.2008	BAB version 4.642
			- Fixed a possible nil initialization bug
			- Fixed a reset of the beginning dynamic combat values
			- Added command option settings menu to the mini DPS/HPS frame
			- Reworked the clicking functionality on the mini DPS/HPS frame to be
			  more like the clicks from FuBar and TitanBar.
			  Left-click & drag to move frame, Left-click to show detailed stats,
			  Right-click to change the settings, Right-shift-click to show detailed stats
		09.05.2008	BAB version 4.641
			- Finish the healing tracking additions, (heals in endoffight & HPS are included)
			- Added functionality to display DPS and HPS in DPS displays in bars and mini window
			- Added commands to display or not display DPS or HPS, (/cs showmdps & /cs showmhps)
			- Improved the in and out of combat flagging
			- Improved onCombatEvent to track fromMe not toMe
			- fixed updates in updateDynCombatInfo by correcting oldesttime setting
		08.27.2008	BAB version 4.640
			- Fixed the miss counters to count the WoW 2.4 combat log misses
			- Fixed HOTs being included in the damage total record
			- Resupport HOTs and DOTs labelling, again the WoW 2.4 combat log change from before
			- Fixed heals being totalled from other as your
			- Lots of code cleanup
			- Detail frame is set to Total upon reset or auto-reset instead of nothing
		06.22.2008	BAB version 4.630
			- Fix in onCombatEvent() to make sure the initial attack is recorded
			- Fix in onCombatEvent() correctly include partial damage from missed,blocked,resisted events
			- Begin addition to track missed damage (attack damage that does not damage target)
		05.11.2008	BAB	version 4.620
			- Added functionality to automatically reset the data when entering a new battle.
			- Rework the slash cmds.
			- Moved the GUI detail options menu to the left side of the frame away from the reset button.
			- Add the automatic reset when entering battle functionality and gui.
		04.19.2008	BAB	version 4.611
			- Fixes to restore the saving off of data.  Addresses a WoW 2.4 change.
		04.17.2008	BAB	version 4.610
			- Changes to resupport TitanBar.  TitanBar is support through outside TitanCombatStats addon.
			- Removed CombatStats.ComParse usage since it is no longer needed
			- Major cleanup and restructuring of this main code file
			- Functions are now defined locally and the public function call are defined at the end
			- Non-english localization string files have been removed since many many string texts
			  where missing from many years of improvements on CombatStats and no one able
			  to contribute string translations to other languages.
		04.11.2008	BAB	version 4.600	
			- Update for WoW version 2.4
			- Completely rework the combat log parsing to use the new combat log
		01.18.2008	BAB	version 4.52
			- Fixed healing record keeping in general.   'incrementSpecialDataRecord' was one place
		11.23.2007	BAB	version 4.511
			- Fixed bug to restore the reset behavior of clear the attack drop down
			  menu also in CombatStats_Reset.
		11.13.2007	BAB	version 4.510
			- Update for WoW version 2.3
			- More memory usage improvements - (very very minor)
		09.25.2007	BAB	version 4.501
			- Update for WoW version 2.2
		09.21.2007	BAB	version 4.50
			- Added functionality to set the maximum updates-per-seconds.
			- Changed CombatStats update frequency so it is now caped at MaxUPS or current FPS
			- Added functionality to position windows back to default. see resetPos()
		09.17.2007	BAB	version 4.492
			- Fixed the call to CombatStats.Utils.NumToBool function.
			- Fixed the FuBar menu gui configuration functionality.
			- Moved Fubar functionality to a separate file.
		09.16.2007	BAB	version 4.491
			- Fixed an initial state nil problem in importDataPre449 function.
		09.15.2007	BAB	version 4.49
			- Memory usage improvements.
			- attackNames storage removed.
			- specialAttackLog storage and usage removed.
			- split common lua code modules into separate support files.
		07.27.2007	BAB	version 4.48
			- Released the 4.47 beta changes
		07.18.2007	BAB	version 4.47 (beta)
			- Attempted a fix for "wowecon" exiting battle sutter
			- Internal improvements for party battle data (end-of-fight)
		07.13.2007	BAB	version 4.46
			- Removed Sea integration because only one simple utility function was being used and
			other newer addon have been released that seem to have replaced it.
			- Unregistering events when they are not needed
			- Replaced Target_Frame function overrides with event
			- Reworked OnUpdate handler as documented in "HOWTO:_Use_OnUpdate_correct"
			- Lots of code cleanup
			- Improvements in attack event parsing, external addon integration, text localization
			- Reworked CombatStatsIB_OnLoad to clean it up and to have all the code in the lua file
			- A rough german translation for the new text additions to CS from version 3.81 on
			[ lots of time work put into looking for strange unexplain problems ]
				[ -- explained with WoW 2.1.3 patch 7/10 release NOTES ] 
				 * Improved the performance of conditional macros, slightly increasing the framerate.
				 * Fixed a crash triggered by many unit frame AddOns.
		07.05.2007	BAB	version 4.45 (beta)	
			- Attempted a fix for non english combat text parsing 
			  [ Supporting CombatStats for non-english WoW version has turn out to be much more difficult then expected ]
			- Improved main event handling - rewrote CombatStats_OnEvent
			- Fixed the the blizzard globally defined attack text conversion for combat parsing
			- Fixed GUI attack menu so it stays sorted correctly
			- THIS BETA RELEASE HAS HAD MINIMUM TESTING AND NO TESTING ON NON-ENGLISH WOW
		07.03.2007	BAB	version 4.44
			- Fixed "in combat" status with PLAYER_REGEN_ENABLED event handling & data start and stop 
			  [ this fixes first strike combat damage and the combat timer start and stop ]
			  [ dynamic run-time DPS and end-of-fight data are now in agreement, except for known discrepancies ]
			- Improved CombatStats_OnUpdate functionality
			- Improved in combat data collection
			- Disabled the overridden ToggleDropDownMenu functionality
			- Fixed show and hide on target functionality (showonlywhentarget option)
			- Changed over to the blizzard globally defined attack text for combat parsing
			  [ combat parsing in all languages should now be supported with this change ]
		06.14.2007	BAB	version 4.43
			- Added FuBar support
			- Fixed a Titan Bar problem
			- Fixed the regular DOT damages not being recorded correctly
		06.08.2007	BAB	version 4.42		
			- Lots of Titan integration improvements -- review the "CombatStatsTB_xxxx" functions
			- Changed over to the blizzard globally defined time format strings in CombatStats_Clock_xx funcs
		06.03.2007	BAB	version 4.41
			- Added the missing 'showttip' command
			- Added the chat command support back for the old CS commands cmd "[on|off]"
		05.23.2007	BAB	version 4.40
			- Update for WoW 2.1.0 interface
		05.21.2007	BAB	version 4.32
			- Fixes to the InfoBar integration for tooltip and right click to display the detailed stats
			- Fixes in the Sea initialization function to handle incompatible Sea versions
			- No longer save off obsolete old saved variables
		05.??.2007	BAB	version 4.31
			- Changed the main default DPS output frame position to no longer depend on the player frame
		05.04.2007	BAB	version 4.3
			- Reformatted the entire statics window to accommodate the high attack values
			- Added the addition drop down menu to allow removal of attack entries & reorder/sort attack menu
			- Removed "Default" attack special ordering in the attack menu
			- Moved many many hard coded strings to the localization.lua file (non-english text variations are still needed)
			- Improved the attack grouping sub menu in the attack menu
			- Many internal code cleanup improvements
			- Added the InfoBar, Sea, Cosmos and Khaos CombatStats changes from http://www.petewood.info/Interface/AddOns/CombatStats/
		04.13.2007	BAB	version 4.01
			- Fixed some nasty wackiness of the initial setting the group field data.
			- Had to add functionality to verify and correct bad data entries.
		04.10.2007	BAB	version 4.00
			- Lots of XML formatting cleanup in the detail frame window
			- Changed attack dropdown menu to a hierarchical dropdown menu to
			  created groups of similar types and shorten the main menu count.
			- Fixed the date time stamp on events so they are now correct between system reboots.
		12.28.2006	BAB	version 3.81
			- Fixed the WoW v2.0 escape key usage of StaticPopup_EscapePressed
		12.19.2006	BAB	version 3.8
			- Fixed to support the WoW v2.0 changes
			- Removed the dependancy on CT_MasterMode because the module is now obsolete
			- Switched the right and left click behavior to be in line with other modes
		
		BAB = Brian Beardmore 
				& released at http://www.realisticsoftware.com/pages/WoWAddons.html

		=-=-=-=-=-=-=-=-=-=-=-=-=-=
		Rev. 3.3	- ReEnabled the option to only show the stats on a mouseclick
		2.16.05		- DPS calculated is now 15 seconds w/no option to change
					- Added end of fight stats option to turn on/off
					- Slash commands will save between sessions when not using w/cosmos
		Rev. 3.2	- Added end of fight statistics
		2.09.05		- Using PLAYER_REGEN_DISABLED ENABLED to tell when you
					  are in/out of combat for end of fight info
		Rev. 3.1	- Lots of fixes to track pet stats
		2.07.05
		Rev. 3.0	- Initial version to track Defensive DPS
		12.xx.04
		Rev 2.03	- Now really really works w/o cosmos
		11.15.04
		Rev 2.02	- Now works w/o cosmos
		11.14.04	- Changed the text display frame
					- Fixed the crit % running out of the frame when it was 100%
					- Put "Default" attack at top of the list
					- Added in a "Total", put it at the bottom of the list
					- UI is now movable
					- UI will update itself while you are attacking if it is visible
					- Fixed a bug where the close "X" button couldn't be pressed
					- Now uses OnEvent and OnUpdate
					- Pet damage is now tracked
					- DOT damage totals are correctly shown
					- Hopefully got rid of bogus entries
		Rev 2.01	- First release w/new UI
		11.05.04
						
	License:
		This program is distributed in the hope that it will be useful,
		but WITHOUT ANY WARRANTY; without even the implied warranty of
		MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

	Note:
		This AddOn's source code is specifically designed to work with
		World of Warcraft's interpreted AddOn system.
]]


-- Debug switch - set to true, to enable debug output for this module
local debug = false;

-------------------------------------------------------------------------------
-- Function Imports
-------------------------------------------------------------------------------
local debugPrint = CombatStats.Utils.DebugPrint
local msgPrint = CombatStats.Utils.MsgPrint
local isTrueValue = CombatStats.Utils.IsTrueValue

local getMissDataForAttack = CombatStats.AttackHistDB.GetMissOverallAttackData

-------------------------------------------------------------------------------
-- Legacy section
-------------------------------------------------------------------------------

--
-- addon constants (used by the internal code only)
--

CombatStats_DPSLen = 30;	-- Calculate DPS over the last 30 seconds

local kDmgEndTotIdx = 1
local kDmgTakenEndTotIdx = 2
local kHealsEndTotIdx = 3
local kDmgTotIdx = 4
local kDmgTakenTotIdx = 5
local kHealsTotIdx = 6

-------------------------------------------------------------------------------
-- local const, variables, and structures
-------------------------------------------------------------------------------
local kAddonName	= "CombatStats"

local _L = {
	TITLE		= GetAddOnMetadata(kAddonName, "Title"),
	VERSION		= GetAddOnMetadata(kAddonName, "Version"),
	AUTHOR		= GetAddOnMetadata(kAddonName, "Author"),
	EMAIL		= GetAddOnMetadata(kAddonName, "X-Email"),
	URL			= GetAddOnMetadata(kAddonName, "X-Website"),
	DATE		= GetAddOnMetadata(kAddonName, "X-Date"),
--	VERSNUM 	= getAddonVersionNumber(),			-- set after function is defined
	WoWintrfc	= 0,
	
	--dpsForBg = "|cffff2020\%.1f|cffffd200\ / |cff20ff20\%.1f",			-- const
	--dpsForSm = "|cffff2020\%.2f|cffffd200\ / |cff20ff20\%.2f",			-- const
	
	--dpsForBg = RED_FONT_COLOR_CODE.."%.1f"..NORMAL_FONT_COLOR_CODE.." / "..GREEN_FONT_COLOR_CODE.."%.1f"..NORMAL_FONT_COLOR_CODE.." / "..BLUE_FONT_COLOR_CODE.."%.1f",	-- const
	--dpsForSm = RED_FONT_COLOR_CODE.."%.2f"..NORMAL_FONT_COLOR_CODE.." / "..GREEN_FONT_COLOR_CODE.."%.2f"..NORMAL_FONT_COLOR_CODE.." / "..BLUE_FONT_COLOR_CODE.."%.2f",	-- const
	
	headerIndexOffset = 12,
		-- garage collection type flags
	kGCafterEOFType		= 1,
	kGCatRestType		= 2,


	myGUID = nil,
	petGUID = nil,			-- reset after a battle
	--tmpStrArr = {},	-- temporary string array, used to avoid creating a bunch of local variables
	bat = {
		yourIdx = 1,				-- FIX THIS
		dpsTakenCalc = 0.0,
		dpsDoneCalc = 0.0,
		hpsDoneCalc = 0.0,
		playersTBL = { },
		--playersBatTots = {
		--	{ dmgEndTot = 0, dmgTakenEndTot = 0, dmgTot = 0, dmgTakenTot = 0 },			-- index 1 is for your information (see yourIdx)
		--},			-- .030-.035 KB per loop called
		playersBatTots = { {0,0,0,0,0,0}, },		-- .029-.033 KB per loop called
		timeStampTaken = {},		-- battle timestamp array for when dmg was taken
		dmgTaken = {},				-- battle damages taken array
		playerTaken = {},			-- player idx in playersTBL that took the battle damages array
		takenCount = 0,				-- player arrays count
		startTakenIdx = 1, 			-- 
		
		timeStampGiven = {},		-- battle timestamp array for when dmg done
		dmgGiven = {},				-- battle damages done array
		playerGiven = {},			-- player idx in playersTBL that gave the battle damages array
		givenCount = 0,				-- player arrays count
		startGivenIdx = 1, 			-- 
		
		timeStampHeal = {},			-- battle timestamp array for when heals where done
		HealGiven = {},				-- battle heals done array
		playerHeal = {},			-- player idx in playersTBL that gave the battle heal array
		givenHealCount = 0,			-- player arrays count
		startHealGivenIdx = 1, 		-- 
	},
	
	bFirstTime = 1,
	bInCombat = 0,
	areActiveRegister = 0,
	
	dirtyAttackMenu = 0,
	timeSinceLastUpdate = 0,
	updateFreq = 0.15,
	lastEnterEventCombatTime = 0,
	lastEnterDmgCombatTime = 0,

	-- last fight data
	lastFight = {
			start 		= 0,
			finish		= 0,
			playerDmg	= 0,
			petDmg		= 0,
			missDmg		= 0,
			healed		= 0,
		},
		
	-- last damage info
	lastdmg = {
			creatureName = "",
			damage = 0,
			damageType = "",
			missDam = 0,
			missType = "",
			spell = "",
			petName = "",
			healP = 0,
		},
		
	-- other addon integration items
	haveInfoBar = false,
	haveKhaos = false,
}

-- Initialize these variables right away
_, _, _, _L.haveInfoBar, _, _, _ = GetAddOnInfo("InfoBar")
_, _, _, _L.haveKhaos, _, _, _ = GetAddOnInfo("Khaos")
_, _, _, _L.WoWintrfc = GetBuildInfo()


-------------------------------------------------------------------------------
-- Saved and Restored Globals
-------------------------------------------------------------------------------

gCS = {}
--[[		see checkLoadedVariables for internal variable initialization
	gCS.currentAttack		=	CS_TEXT_NA
	specialAttacks[CS_ATTACK_TOTAL_LABEL].totalHits 		=	0
	...
]]
	--	Configuration variable storage
CombatStats_Config = {}
--[[		see checkLoadedVariables for internal variable initialization
	CombatStats_Config.CombatStats_OnOff = 1
	CombatStats_Config.CombatStats_HideOnNoTarget = 0
	CombatStats_Config.CombatStats_EndOfFight = 0
	CombatStats_Config.CombatStats_UseMouseOver = 0
	CombatStats_Config.CombatStats_Hide = 0
	CombatStats_Config.CombatStats_ShowTtip = 1
	CombatStats_Config.CombatStats_AttackMenuSort = 0
	CombatStats_Config.CombatStats_AttackMenu1stGrping = 1
	CombatStats_Config.CombatStats_AutoBattleReset = 0
	CombatStats_Config.CombatStats_MaxUpPS = 30
]]

-------------------------------------------------------------------------------
-- Data Members
-------------------------------------------------------------------------------

-------------------------------------------------------------------------------
-- Constants
-------------------------------------------------------------------------------

-------------------------------------------------------------------------------
-- Uses
-------------------------------------------------------------------------------
	
-------------------------------------------------------------------------------
-- Function Prototypes - Local
-------------------------------------------------------------------------------

-- Setup Funcs
	local checkLoadedVariables
	local resetMiniDPSFormat
-- Event Handling Funcs
	local combatstats_onLoad
	local combatstats_onUpdate
	local combatstats_onEvent
	local onCombatEvent
	local chatCommandHandler
	
	local registerAllEvents
	local updateWhenActiveRegistery
	local updateWhenRestingRegistery
	local registerWhenActiveEvents
	local unregisterWhenActiveEvents
	-- CombatStatsText
	local combatStatsText_OnClick
	local combatStatsText_OnEnter
-- Core Functionality Funcs
	local reset
	local resetPos
	local information
	local getCurrentCombatData
	local getFormattedCurrentCombatData
	local getUpdateFreq
	local getDynDPSformatStr
	local getCurrentDynDPSformatStr
-- Support Functionality Funcs
	local updateVisibility
	local showLastFightDPS
	local enterCombatMode
	local endCombatMode
	local chkIfCombatModeShouldEnd
	local getSwingType
	local getInternalMissType
	local getInternalHitType
	
	local setDynCombatTexts
	local updateDynCombatTexts
	local updateDynCombatInfo
	local chkGarbage

	local getAddonVersionNumber
	local chatCommandUsage
-- Damage Done and Taken Funcs
	local addDPSEntry
	local addDefDPSEntry
	local addHealEntry
	local addSpecialData
	local incrementSpecialDataRecord
	local subtractFromSpecialDataTotals
-- Settings Accessors Funcs
	local setEnable
	local setTarget
	local setMouseOver
	local setEndFight
	local setHide
	local setShowTtip
	local setShowDPSinMini
	local setShowHPSinMini
	local setAutoBatReset
	local updateFramePer
	local setMaxUPS
-- Utility Funcs
	local arrTBLItemCount
	local dumpInternalStats
	local dumpConfigVariables
	local dumpArrayString

-- InfoBar Funcs
	local combatStatsIB_OnLoad
	local combatStatsIB_OnClick
	local combatStatsIB_OnEnter
	local combatStatsIB_Tooltip
-- Khaos Funcs
	local registerKhaos


------------------------------------------------------------
-- The Functional Code
------------------------------------------------------------

--
-- Setup Funcs
--

function checkLoadedVariables()
	local 	aCounter
	local	removeCount = 0
	
	--[[ add missing config record fields ]]
	CombatStats_Config.CombatStats_OnOff = isTrueValue(CombatStats_Config.CombatStats_OnOff or true)
	-- toggle twice back to the correct state (done to handle the event register)
	setEnable()
	setEnable()
	
	CombatStats_Config.CombatStats_HideOnNoTarget = isTrueValue(CombatStats_Config.CombatStats_HideOnNoTarget or false)
	CombatStats_Config.CombatStats_EndOfFight = isTrueValue(CombatStats_Config.CombatStats_EndOfFight or false)
	CombatStats_Config.CombatStats_UseMouseOver = isTrueValue(CombatStats_Config.CombatStats_UseMouseOver or false)
	CombatStats_Config.CombatStats_Hide = isTrueValue(CombatStats_Config.CombatStats_Hide or false)
	CombatStats_Config.CombatStats_ShowTtip = isTrueValue(CombatStats_Config.CombatStats_ShowTtip or true)
	CombatStats_Config.CombatStats_ShowDPSinMini = isTrueValue(CombatStats_Config.CombatStats_ShowDPSinMini or true)
	CombatStats_Config.CombatStats_ShowHPSinMini = isTrueValue(CombatStats_Config.CombatStats_ShowHPSinMini or false)
	CombatStats_Config.CombatStats_AttackMenuSort = CombatStats_Config.CombatStats_AttackMenuSort or 0
	CombatStats_Config.CombatStats_AttackMenu1stGrping = CombatStats_Config.CombatStats_AttackMenu1stGrping or 1
	
	CombatStats_Config.CombatStats_AutoBattleReset = isTrueValue(CombatStats_Config.CombatStats_AutoBattleReset or false)
	--		CombatStats.AttackHistDB.setGrpSortType(CombatStats_Config.CombatStats_AttackMenu1stGrping);		-- FIX ME!!!
	CombatStats_Config.CombatStats_GC = CombatStats_Config.CombatStats_GC or 0
	CombatStats_Config.CombatStats_MaxUpPS = CombatStats_Config.CombatStats_MaxUpPS or 30
	gCS.currentAttack = gCS.currentAttack or CS_TEXT_NA
	
	if (gCS.dataVersion and (gCS.dataVersion < 464)) then
		msgPrint("Old CombatStats data is being reset")
		specialAttackLog = nil;			-- no longer needed or used
		attackNames = nil;				-- no longer needed or used
		gCS.specialsCount = nil;		-- no longer needed or used
	end
	
	resetMiniDPSFormat()
	
	gCS.dataVersion = _L.VERSNUM
end


function resetMiniDPSFormat()
	_L.dpsForSm = getCurrentDynDPSformatStr(8)
	_L.dpsForBg = getCurrentDynDPSformatStr(20)
end

--
-- Event Handling Funcs
--

local function combatstats_onLoad(self)
	
	if ( not Khaos ) then
		-- Standalone
		SLASH_CSSLASH1 = "/combatstats"
		SLASH_CSSLASH2 = "/cs"
		SlashCmdList["CSSLASH"] = CombatStats_ChatCommandHandler
		
		if ( _L.TITLE and _L.VERSION ) then
			DEFAULT_CHAT_FRAME:AddMessage( "|cffffff00".._L.TITLE..", v".._L.VERSION.."|r loaded. Type |cffffff00"..SLASH_CSSLASH1.."|r for help." )
		else
			DEFAULT_CHAT_FRAME:AddMessage( "|cffffff00"..kAddonName.."|r loaded. Type |cffffff00"..SLASH_CSSLASH1.."|r for help." )
		end
	else
		-- Special OnLoad for "Khaos"
		registerKhaos()
	end
	
	-- setup for InfoBar if its being used
	if (InfoBarFrame and IB_TextFrame) then
		local tmpFrame = CreateFrame("frame", "CombatStatsIBFrame", "InfoBarFrame", "IB_TextFrame")
		tmpFrame.info = 
			{
				name=_L.TITLE,
				version=_L.VERSNUM,
				update=combatstats_onUpdate,
				tooltip=combatStatsIB_Tooltip,
			}
		tmpFrame:SetScript("OnEnter", CombatStatsIB_OnEnter)
		tmpFrame:SetScript("OnMouseDown", CombatStatsIB_OnClick)

		tmpFrame:Show()
		if(tmpFrame:IsVisible()) then
			tmpFrame:SetText(CS_MINIFRAME_DEFAULT_TEXT)
		end
	end
	
	registerAllEvents(self)
	tinsert(UISpecialFrames, "CombatStatsDataFrame")
	CombatStatsText:SetText(CS_MINIFRAME_DEFAULT_TEXT)
	updateVisibility()
end


local function combatstats_onUpdate(self, elapsed)
	local curFPS
	_L.timeSinceLastUpdate = _L.timeSinceLastUpdate + elapsed
	
	-- Check to see if its time to update
	if (_L.timeSinceLastUpdate > _L.updateFreq) then
		if (_L.bInCombat == 1) then
			updateDynCombatInfo(GetTime())
			chkIfCombatModeShouldEnd()
		end
		_L.timeSinceLastUpdate = 0
		
		curFPS = GetFramerate()
		if (curFPS > CombatStats_Config.CombatStats_MaxUpPS) then curFPS = CombatStats_Config.CombatStats_MaxUpPS end
		if (curFPS < 1) then curFPS = 1 end
		_L.updateFreq = 1/curFPS	-- dynamically change the updates frequency depending on the FPS available
	end
end


function combatstats_onEvent(self, event, ...)
	local sysdatetime = time()
	local tickCount = GetTime()	
	--
	-- Event Handler
	-- the event switch order is based on most used and time critial
	--
	
	--if ((event == "PLAYER_REGEN_ENABLED") or (event == "PLAYER_LEAVE_COMBAT" )) then
	if (event == "PLAYER_REGEN_ENABLED") then
						-- note it appears PLAYER_LEAVE_COMBAT triggers after a single target falls
		-- switching from in combat to out of combat
		endCombatMode(event)
		return
		
	elseif ((event == "PLAYER_REGEN_DISABLED") or (event == "PLAYER_ENTER_COMBAT")) then
		enterCombatMode(event)
		return
	
	--
	-- System type events
	--
	elseif ((event == "PLAYER_FOCUS_CHANGED") or (event == "PLAYER_TARGET_CHANGED")) then
		-- replacement event for the function TargetFrame_OnShow, TargetFrame_OnHide overrides
		-- the newer PLAYER_FOCUS_CHANGED event should covers PLAYER_TARGET_CHANGED - but it does not seem to
		updateVisibility()
		return
		
	elseif (event == "VARIABLES_LOADED") then
		CombatStats.AttackHistDB.Load()
		
		checkLoadedVariables()
		CombatStats.DetailFrame.SetDirtyAttackMenu()
		updateWhenActiveRegistery(self)
		
		updateVisibility()
		
		-- no longer need to track this event
		self:UnregisterEvent("VARIABLES_LOADED")
		return
	
	elseif (event == "PLAYER_UPDATE_RESTING") then
		chkGarbage(_L.kGCatRestType)
		return

	elseif (event == "PLAYER_LEAVING_WORLD") then
		CombatStats.AttackHistDB.Save()
		return

	-- in combat events
	elseif (event == "COMBAT_LOG_EVENT_UNFILTERED") then
		--	Don't do anything else at this point unless
		--	combat stats is turned on (should never have to use this check if if register and unregister code is working)
		--
		if ( CombatStats_Config.CombatStats_OnOff ) then
			onCombatEvent(CombatLogGetCurrentEventInfo())
		end
		
		return
		
	else
		return
	end
	
end


function onCombatEvent(...)

	local timestamp, eventtype, hideCaster, srcGUID, srcName, srcFlags, srcRaidFlags, dstGUID, dstName, dstFlags, dstRaidFlags = select(1, ...);
	local msgTxt = ""
	local sysdatetime = time()
	local tickCount = GetTime()
	local fromMe = false
	local fromPet = false
	local petEvent = false
	-- Spell standard order
	local spellId, spellName, spellSchool
	-- Damage standard order
	local amount, overkill, school, resisted, blocked, absorbed, critical, glancing, crushing, isOffHand
	-- Miss argument order
	local missType
	--
	local missPercentage
	
	--msgPrint("timestamp: "..(timestamp or "nil").."sysdatetime: "..(sysdatetime or "nil").."tickCount: "..(tickCount or "nil"))
	-- Note sysdatetime is a integer rounded value of timestamp

	if (_L.myGUID == nil) then
		if ((srcName ~= nil) and (srcName == UnitName("player"))) then
			_L.myGUID = srcGUID
		elseif ((dstName ~= nil) and (dstName == UnitName("player"))) then
			_L.myGUID = dstGUID
		end
	end
	if (_L.petGUID == nil) then
		if ((srcName ~= nil) and (srcName == UnitName("pet"))) then
			_L.petGUID = srcGUID
		elseif ((dstName ~= nil) and (dstName == UnitName("pet"))) then
			_L.petGUID = dstGUID
		end
	end
	
	fromMe = ((_L.myGUID ~= nil) and ( srcGUID == _L.myGUID ))
	fromPet = ((_L.petGUID ~= nil) and ( srcGUID == _L.petGUID ))
	
	if (( srcGUID == _L.myGUID ) or ( dstGUID == _L.myGUID ) or
		( srcGUID == _L.petGUID ) or ( dstGUID == _L.petGUID )) then

		petEvent = ( (_L.petGUID ~= nil) and (( srcGUID == _L.petGUID ) or ( dstGUID == _L.petGUID )) )
		
		_L.lastdmg = {						-- empty last attack information
			creatureName = (dstName or ""),
			damage = 0,
			damageType = "",
			missDam = 0,
			missType = "",
			spell = CS_ATTACK_GEN_DEF_LABEL,
			petName = "",
			healP = 0,
		}
		missPercentage = 0
		if ((eventtype == "SWING_DAMAGE") or (eventtype == "DAMAGE_SPLIT")) then			-- special handling of DAMAGE_SPLIT unknown it this time
			amount, overkill, school, resisted, blocked, absorbed, critical, glancing, crushing, isOffHand = select(_L.headerIndexOffset, ...);
			
			_L.lastdmg.damageType, _L.lastdmg.missType = getInternalHitType(resisted, blocked, absorbed, critical, glancing, crushing)
			_L.lastdmg.missDam = tonumber((resisted or 0) + (blocked or 0) + (absorbed or 0))
			if ( (_L.lastdmg.damageType == CombatStats.kHitType) or (_L.lastdmg.damageType == CombatStats.kCritType) ) then
				_L.lastdmg.spell = CS_ATTACK_GEN_DEF_LABEL
				_L.lastdmg.damage = tonumber(amount)
			end
			
		elseif (eventtype == "SWING_MISSED") then
			missType, _, _L.lastdmg.missDam = select(_L.headerIndexOffset, ...)
			_L.lastdmg.missType = getInternalMissType(missType)
			missPercentage = 1
			debugPrint("in SWING_MISSED, missType: "..(missType or "nil")..", myType: "..(_L.lastdmg.missType or "nil"))
			
		elseif (eventtype == "RANGE_DAMAGE") then
			-- Shots are spells, technically
			spellId, spellName, spellSchool, amount, overkill, school, resisted, blocked, absorbed, critical, glancing, crushing = select(_L.headerIndexOffset, ...)
			
			_L.lastdmg.spell = tostring(spellName or CS_ATTACK_GEN_DEF_LABEL)
			_L.lastdmg.damageType, _L.lastdmg.missType = getInternalHitType(resisted, blocked, absorbed, critical, glancing, crushing)
			_L.lastdmg.missDam = tonumber((resisted or 0) + (blocked or 0) + (absorbed or 0))
			if ( (_L.lastdmg.damageType == CombatStats.kHitType) or (_L.lastdmg.damageType == CombatStats.kCritType) ) then
				_L.lastdmg.damage = tonumber(amount)
			end
			
		elseif (eventtype == "RANGE_MISSED") then
			-- Shots are spells, technically
			spellId, spellName, spellSchool, missType = select(_L.headerIndexOffset, ...)
			
			_L.lastdmg.spell = tostring(spellName or CS_ATTACK_GEN_DEF_LABEL)
			_L.lastdmg.missType = getInternalMissType(missType)
			missPercentage = 1
		
		elseif (strsub(eventtype, 1, 6) == "SPELL_" ) then
			if ((eventtype == "SPELL_DAMAGE") or (eventtype == "SPELL_PERIODIC_DAMAGE") or (eventtype == "SPELL_BUILDING_DAMAGE")) then
				spellId, spellName, spellSchool, amount, overkill, school, resisted, blocked, absorbed, critical, glancing, crushing, isOffHand = select(_L.headerIndexOffset, ...)
				
				_L.lastdmg.spell = tostring(spellName or CS_ATTACK_GEN_DEF_LABEL)
				_L.lastdmg.damageType, _L.lastdmg.missType = getInternalHitType(resisted, blocked, absorbed, critical, glancing, crushing)
				_L.lastdmg.missDam = tonumber((resisted or 0) + (blocked or 0) + (absorbed or 0))
				if ( (_L.lastdmg.damageType == CombatStats.kHitType) or (_L.lastdmg.damageType == CombatStats.kCritType) ) then
					_L.lastdmg.damage = tonumber(amount)
					if ((_L.lastdmg.damageType == CombatStats.kHitType) and (eventtype == "SPELL_PERIODIC_DAMAGE")) then
						_L.lastdmg.damageType = CombatStats.kDotType end
					if (_L.lastdmg.damage == 0) then _L.lastdmg.damageType = "" end
				end
				
			elseif ((eventtype == "SPELL_MISSED") or (eventtype == "SPELL_PERIODIC_MISSED")) then
				-- SPELL_PERIODIC_MISSED does not have _L.lastdmg.missDam returned except when type is ABSORB
				spellId, spellName, spellSchool, missType, _L.lastdmg.missDam = select(_L.headerIndexOffset, ...)
				
				_L.lastdmg.spell = tostring(spellName or CS_ATTACK_GEN_DEF_LABEL)
				_L.lastdmg.missType = getInternalMissType(missType)
				missPercentage = 1
				
			elseif ((eventtype == "SPELL_HEAL") or (eventtype == "SPELL_PERIODIC_HEAL") or (eventtype == "SPELL_BUILDING_HEAL")) then
				spellId, spellName, spellSchool, amount, overhealing, _, critical = select(_L.headerIndexOffset, ...)
				
				_L.lastdmg.healP = amount
				if (critical) then _L.lastdmg.damageType = CombatStats.kCritType
				else _L.lastdmg.damageType = CombatStats.kHitType end
				_L.lastdmg.spell = tostring(spellName or CS_ATTACK_GEN_DEF_LABEL)
				if ((_L.lastdmg.damageType == CombatStats.kHitType) and (eventtype == "SPELL_PERIODIC_HEAL")) then
					_L.lastdmg.damageType = CombatStats.kHotType end
					
			elseif ( (eventtype == "SPELL_ENERGIZE") or (eventtype == "SPELL_SUMMON") or
				(strsub(eventtype, 1, 11) == "SPELL_AURA_" ) or (strsub(eventtype, 1, 11) == "SPELL_CAST_" ) ) then
					-- ignoring these events
					return 0				-- ####### returning at this point
			else
				debugPrint("### eventtype (spell: "..eventtype.." was skipped")
				return 0				-- ####### returning at this point
			end
			
		else
			--msgPrint("### eventtype: "..eventtype.." was skipped")
			--[[
			for i = 1, select("#", ...) do
				--msgPrint(""..i..") "..(select(i, ...) or "nil"))
				msgTxt = msgTxt.." "..i..") "..(select(i, ...) or "nil")
			end
			msgPrint(msgTxt)
			]]
			return 0				-- ####### returning at this point
		end
		
		if (_L.lastdmg.healP == 0) then
			-- NOT HEALING
			if (_L.lastdmg.damageType ~= "") then
				if (petEvent) then
					if (fromPet) then
						enterCombatMode()
						addSpecialData(CS_ATTACK_PET_PRE_LABEL.._L.lastdmg.spell,_L.lastdmg.damageType,_L.lastdmg.damage,sysdatetime,false,true)
						_L.lastFight.playerDmg = _L.lastFight.playerDmg + _L.lastdmg.damage
						_L.lastFight.petDmg = _L.lastFight.petDmg + _L.lastdmg.damage
						
						-- note all misses should be handled in addSpecialDataMiss
					--else
					--	--addSpecialData(CS_ATTACK_PET_PRE_LABEL..CS_ATTACK_DEFENSE_LABEL,_L.lastdmg.damageType,_L.lastdmg.damage,sysdatetime,false,true)
					--	addSpecialData(CS_ATTACK_PET_PRE_LABEL..CS_ATTACK_DEFENSE_LABEL,_L.lastdmg.damageType,0,sysdatetime,false,true)
					end
				else
					if (fromMe) then
						enterCombatMode()
						addSpecialData("".._L.lastdmg.spell,_L.lastdmg.damageType,_L.lastdmg.damage,sysdatetime,false,false)
						_L.lastFight.playerDmg = _L.lastFight.playerDmg + _L.lastdmg.damage
						
						-- note all misses should be handled in addSpecialDataMiss
					--else
					--	--addSpecialData(CS_ATTACK_DEFENSE_LABEL,_L.lastdmg.damageType,_L.lastdmg.damage,sysdatetime,false,false)
					--	addSpecialData(CS_ATTACK_DEFENSE_LABEL,_L.lastdmg.damageType,0,sysdatetime,false,false)
					end
				end
				
				if (_L.lastdmg.damage ~= 0) then
					enterCombatMode()
					if ((fromMe) or (fromPet)) then
						addDPSEntry(tickCount, CombatStats.kYourType, _L.lastdmg.damage)
					else
						-- someday may want to check if toMe and toPet
						addDefDPSEntry(tickCount, CombatStats.kYourType,_L.lastdmg.damage)
					end
				end
			end
			
			if (_L.lastdmg.missDam == nil or _L.lastdmg.missDam == false) then _L.lastdmg.missDam = 0 end
			if ((_L.lastdmg.missType ~= "") and (_L.lastdmg.missDam or missPercentage)) then
				if (petEvent) then
					if (fromPet) then
						enterCombatMode()
						addSpecialDataMiss(CS_ATTACK_PET_PRE_LABEL.._L.lastdmg.spell,_L.lastdmg.missType,_L.lastdmg.damage,sysdatetime,false,true,_L.lastdmg.missDam,missPercentage)
						addSpecialDataMiss(CS_ATTACK_TOTAL_LABEL,_L.lastdmg.missType,_L.lastdmg.damage,sysdatetime,false,true,_L.lastdmg.missDam,missPercentage)
						
						if (missPercentage) then
							_, _, amount, _ = getMissDataForAttack(_L.lastdmg.spell, _L.lastdmg.missType)
							-- note blizz does not return how much dmg was completely missed so an avg amount is used
							_L.lastFight.missDmg = _L.lastFight.missDmg + _L.lastdmg.missDam + (missPercentage * tonumber(amount))
						else
							_L.lastFight.missDmg = _L.lastFight.missDmg + _L.lastdmg.missDam
						end
					end
				else
					if (fromMe) then
						enterCombatMode()
						addSpecialDataMiss("".._L.lastdmg.spell,_L.lastdmg.missType,_L.lastdmg.damage,sysdatetime,false,false,_L.lastdmg.missDam,missPercentage)
						addSpecialDataMiss(CS_ATTACK_TOTAL_LABEL,_L.lastdmg.missType,_L.lastdmg.damage,sysdatetime,false,false,_L.lastdmg.missDam,missPercentage)
						if (missPercentage) then
							_, amount, _ = getMissDataForAttack(_L.lastdmg.spell, _L.lastdmg.missType)
							-- note blizz does not return how much dmg was completely missed so an avg amount is used
							_L.lastFight.missDmg = _L.lastFight.missDmg + _L.lastdmg.missDam + (missPercentage * tonumber(amount))
						else
							_L.lastFight.missDmg = _L.lastFight.missDmg + _L.lastdmg.missDam
						end
					end
				end
			end
		else
			-- HEALING
			if (petEvent) then
				if (fromPet) then
					addHealEntry(tickCount, CombatStats.kYourType, _L.lastdmg.healP)
					-- pet healing is not separate at this time and I do not think there is a pet that heals at this time
					addSpecialData(CS_ATTACK_HEAL_PRE_LABEL.._L.lastdmg.spell,_L.lastdmg.damageType,_L.lastdmg.healP,sysdatetime,true,false)
					_L.lastFight.healed = _L.lastFight.healed + _L.lastdmg.healP
				end
			else
				if (fromMe) then
					addHealEntry(tickCount, CombatStats.kYourType, _L.lastdmg.healP)
					addSpecialData(CS_ATTACK_HEAL_PRE_LABEL.._L.lastdmg.spell,_L.lastdmg.damageType,_L.lastdmg.healP,sysdatetime,true,false)
					_L.lastFight.healed = _L.lastFight.healed + _L.lastdmg.healP
				end
			end
			
		end
		
	end
end


function CombatStatsSettingsMenuInitialize(self)
	local dropdown = self:GetParent()
	--local dropdown = nil
	
	if (CombatStats_Config.CombatStats_OnOff)
		then CombatStats.Utils.AppendMenuChkItem(CS_CMD_ENABLE_OLD_TEXT,0,setEnable,dropdown)
		else CombatStats.Utils.AppendMenuItem(CS_CMD_ENABLE_OLD_TEXT,0,setEnable,dropdown)
	end
	if (CombatStats_Config.CombatStats_HideOnNoTarget)
		then CombatStats.Utils.AppendMenuChkItem(CS_CMD_TARGET_OLD_TEXT,0,setTarget,dropdown)
		else CombatStats.Utils.AppendMenuItem(CS_CMD_TARGET_OLD_TEXT,0,setTarget,dropdown)
	end
	if (CombatStats_Config.CombatStats_UseMouseOver)
		then CombatStats.Utils.AppendMenuChkItem(CS_CMD_MOUSEOVER_OLD_TEXT,0,setMouseOver,dropdown)
		else CombatStats.Utils.AppendMenuItem(CS_CMD_MOUSEOVER_OLD_TEXT,0,setMouseOver,dropdown)
	end
	if (CombatStats_Config.CombatStats_EndOfFight)
		then CombatStats.Utils.AppendMenuChkItem(CS_CMD_ENDFLIGHT_OLD_TEXT,0,setEndFight,dropdown)
		else CombatStats.Utils.AppendMenuItem(CS_CMD_ENDFLIGHT_OLD_TEXT,0,setEndFight,dropdown)
	end
	if (CombatStats_Config.CombatStats_Hide)
		then CombatStats.Utils.AppendMenuChkItem(CS_CMD_HIDE_OLD_TEXT,0,setHide,dropdown)
		else CombatStats.Utils.AppendMenuItem(CS_CMD_HIDE_OLD_TEXT,0,setHide,dropdown)
	end
	if (CombatStats_Config.CombatStats_ShowDPSinMini)
		then CombatStats.Utils.AppendMenuChkItem(CS_CMD_SHOWDPSMINI_OLD_TEXT,0,setShowDPSinMini,dropdown)
		else CombatStats.Utils.AppendMenuItem(CS_CMD_SHOWDPSMINI_OLD_TEXT,0,setShowDPSinMini,dropdown)
	end
	if (CombatStats_Config.CombatStats_ShowHPSinMini)
		then CombatStats.Utils.AppendMenuChkItem(CS_CMD_SHOWHPSMINI_OLD_TEXT,0,setShowHPSinMini,dropdown)
		else CombatStats.Utils.AppendMenuItem(CS_CMD_SHOWHPSMINI_OLD_TEXT,0,setShowHPSinMini,dropdown)
	end
	if (CombatStats_Config.CombatStats_ShowTtip)
		then CombatStats.Utils.AppendMenuChkItem(CS_CMD_TOOLTIP_OLD_TEXT,0,setShowTtip,dropdown)
		else CombatStats.Utils.AppendMenuItem(CS_CMD_TOOLTIP_OLD_TEXT,0,setShowTtip,dropdown)
	end
	if (CombatStats_Config.CombatStats_AutoBattleReset)
		then CombatStats.Utils.AppendMenuChkItem(CS_CMD_RESETONBAT_TEXT,0,setAutoBatReset,dropdown)
		else CombatStats.Utils.AppendMenuItem(CS_CMD_RESETONBAT_TEXT,0,setAutoBatReset,dropdown)
	end
	
	CombatStats.Utils.AppendMenuItemSeparator("- - - - -",dropdown);
	CombatStats.Utils.AppendMenuItem(CS_CMD_SHOWSTAT_TEXT,0,CombatStatsText_ShowFrame,dropdown)
	
	CombatStats.Utils.AppendMenuItemSeparator("- - - - -",dropdown);
	CombatStats.Utils.AppendMenuItem(CS_CMD_RESET_OLD_TEXT,0,reset,dropdown)
	CombatStats.Utils.AppendMenuItem(CS_CMD_RESETWIN_TEXT,0,resetPos,dropdown)
	CombatStats.Utils.AppendMenuItem(CS_CMD_SETMAXUPS_TEXT,0,setMaxUPS,dropdown)
	
	CombatStats.Utils.FixLvlDropMenuButtons()
end


function chatCommandHandler(msg)
	--msgPrint("CombatStats_ChatCommandHandler("..asText(msg)..")")
	local aResult = nil
	
	if( not msg or msg == "" ) then
		chatCommandUsage()
	else
		local msg = string.lower(msg)
		local onoff = nil
		local firsti, lasti, command, setStr = string.find (msg, "(%w+) ([%w%.]+)")
		
		if (firsti ~= nil) then
			setStr = string.lower(setStr)
			if ((setStr == "yes") or (setStr == "y") or (setStr == "on")) then
				onoff = 1
			elseif ((setStr == "no") or (setStr == "n") or (setStr == "off")) then
				onoff = 0
			end
		else
			command = msg
		end
		
		
		-- handle non option commands
		if(command == CS_CMD_SHOWSTAT) then
			CombatStatsText_ShowFrame()
			return
		elseif(command == CS_CMD_RESETWIN) then
			resetPos()
			return
		elseif((command == CS_CMD_SETMAXUPS) and (setStr == nil)) then
			setMaxUPS()			-- display the gui dialog because a value was not entered
			return
		-- handle option commands
		elseif((command == CS_CMD_SETMAXUPS) and (setStr ~= nil)) then
			if (CombatStats.Dialogs.ChkSet_UpPSValue(setStr)) then
				msgPrint("cs option "..command.." was set to "..CombatStats_Config.CombatStats_MaxUpPS)
			else
				msgPrint("cs option "..command.." was NOT set")
			end
			return
		elseif(command == CS_CMD_ENABLE) then
			aResult = setEnable(onoff)
		elseif(command == CS_CMD_TARGET) then
			aResult = setTarget(onoff)
		elseif(command == CS_CMD_MOUSEOVER) then
			aResult = setMouseOver(onoff)
		elseif(command == CS_CMD_ENDFLIGHT) then
			aResult = setEndFight(onoff)
		elseif(command == CS_CMD_HIDE) then
			aResult = setHide(onoff)
		elseif(command == CS_CMD_SHOWDPSMINI) then
			aResult = setShowDPSinMini(onoff)
		elseif(command == CS_CMD_SHOWHPSMINI) then
			aResult = setShowHPSinMini(onoff)
		elseif(command == CS_CMD_TTIP) then
			aResult = setShowTtip(onoff)
		elseif(command == CS_CMD_RESET) then
			aResult = reset(onoff)
			updateVisibility()
			return
		else
			chatCommandUsage()
			return
		end
		
		if (aResult ~= nil) then
			if (aResult) then
				msgPrint("cs option "..command.." is on")
				updateVisibility()
			else
				msgPrint("cs option "..command.." is off")
				updateVisibility()
			end
		end
		
	end
end


function registerAllEvents(self)
	-- registerWhenActiveEvents is called after the setting are loaded if needed
	self:RegisterEvent("VARIABLES_LOADED")
	self:RegisterEvent("PLAYER_LEAVING_WORLD")
	--self:RegisterEvent("PLAYER_FOCUS_CHANGED")
end

function updateWhenActiveRegistery(self)
	if ( CombatStats_Config.CombatStats_OnOff ) then
		registerWhenActiveEvents(self)
		updateWhenRestingRegistery(self)
	else
		unregisterWhenActiveEvents(self)
	end
end


function updateWhenRestingRegistery(self)
	if (CombatStats_Config.CombatStats_GC == _L.kGCatRestType) then
		self:RegisterEvent("PLAYER_UPDATE_RESTING")
	else
		self:UnregisterEvent("PLAYER_UPDATE_RESTING")
	end
end
		
		
function registerWhenActiveEvents(self)
	--msgPrint("registerWhenActiveEvents")
	if (_L.areActiveRegister == 1) then return end
	
	--self:RegisterEvent("PLAYER_FOCUS_CHANGED")
	self:RegisterEvent("PLAYER_TARGET_CHANGED")

	--self:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED", "onCombatEvent")
	self:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
	self:RegisterEvent("PLAYER_REGEN_ENABLED")
	self:RegisterEvent("PLAYER_REGEN_DISABLED")
	self:RegisterEvent("PLAYER_ENTER_COMBAT")
	self:RegisterEvent("PLAYER_LEAVE_COMBAT")
	
	_L.areActiveRegister = 1
end


function unregisterWhenActiveEvents(self)
	--msgPrint("unregisterWhenActiveEvents")
	if (_L.areActiveRegister == 0) then return end
	
	self:UnregisterEvent("PLAYER_FOCUS_CHANGED")
	self:UnregisterEvent("PLAYER_TARGET_CHANGED")

	self:UnregisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
	self:UnregisterEvent("PLAYER_REGEN_ENABLED")
	self:UnregisterEvent("PLAYER_REGEN_DISABLED")
	self:UnregisterEvent("PLAYER_ENTER_COMBAT")
	self:UnregisterEvent("PLAYER_LEAVE_COMBAT")
	
	--self:UnregisterEvent("PLAYER_UPDATE_RESTING")
	
	_L.areActiveRegister = 0
end


local function combatStatsText_OnClick()
	if (CombatStatsDataFrame:IsVisible()) then
		CombatStatsDataFrame:Hide()
	else
		if (not CombatStats_Config.CombatStats_UseMouseOver) then
			CombatStatsText_ShowFrame()
		end
	end
end


function combatStatsText_OnEnter()
	if (CombatStats_Config.CombatStats_UseMouseOver) then
		CombatStatsText_ShowFrame()
	end
end

--
-- Core Functionality Funcs
--

function reset()
	_L.bat.takenCount = 0
	_L.bat.startTakenIdx = 1
	_L.bat.givenCount = 0
	_L.bat.startGivenIdx = 1
	_L.bat.givenHealCount = 0
	_L.bat.startHealGivenIdx = 1
	-- and the arrays connected to them
	
	_L.bat.dpsTakenCalc = 0.0
	_L.bat.dpsDoneCalc = 0.0
	_L.bat.hpsDoneCalc = 0.0
	--_L.bat.playersBatTots = {
	--	{ dmgEndTot = 0, dmgTakenEndTot = 0, dmgTot = 0, dmgTakenTot = 0 },			-- index 1 is for your information (see yourIdx)
	--}
	_L.bat.playersBatTots = { {0,0,0,0,0,0}, }
	
	CombatStats.AttackHistDB.Clear()
	
	CombatStatsText:SetText(CS_MINIFRAME_DEFAULT_TEXT)
	CombatStatsText_ResetFrameGUI()
	--specialAttacks[CS_ATTACK_TOTAL_LABEL] = CombatStats_New_SpecialAttack(CS_ATTACK_TOTAL_LABEL, 0)
	local attackRecord = CombatStats.AttackHistDB.GetAttackRec(CS_ATTACK_TOTAL_LABEL,true,true)
	if (attackRecord) then
		gCS.currentAttack = CS_ATTACK_TOTAL_LABEL
		CombatStats_UpdateDetails(gCS.currentAttack)
		CombatStatsAttackDropDown_SetText(gCS.currentAttack)
	else
		gCS.currentAttack = CS_TEXT_NA
	end
end


function resetPos()
	local frame;
	frame = CombatStatsFrame;
	-- reset the small DPS frame
	if (frame ~= nil) then
		frame:StopMovingOrSizing();
		frame.isMoving = false;
		frame:ClearAllPoints();
		frame:SetPoint("TOPLEFT", "UIParent", "TOPLEFT", 240, -25);
	
		-- Make sure the window is visible, also.
		setTarget(0);
		setHide(0);
		frame:Show();
	--else
		--msgPrint("resetPos : Error getting small DPS frame.");
	end
	
	frame = CombatStatsDataFrame;
	-- reset the detail stats frame
	if (frame ~= nil) then
		frame:StopMovingOrSizing();
		frame.isMoving = false;
		frame:ClearAllPoints();
		frame:SetPoint("TOPLEFT", "UIParent", "TOPLEFT", 30, -150);
	
		-- Make sure the window is visible, too.
		CombatStatsText_ShowFrame();
	--else
		--msgPrint("resetPos : Error getting detail stats frame.");
	end
	
end


function information()
	return _L.TITLE, _L.VERSION, _L.VERSNUM, _L.DATE
end


function getCurrentCombatData()
	return (_L.bat.dpsTakenCalc or 0.0), (_L.bat.dpsDoneCalc or 0.0), (_L.bat.hpsDoneCalc or 0.0)
end

function getFormattedCurrentCombatData()
	-- keep fall back settings in sync with getDynDPSformatStr()
	local includeDps = (CombatStats_Config.CombatStats_ShowDPSinMini)
	local includeHps = (CombatStats_Config.CombatStats_ShowHPSinMini)
	local greatestValue = 0.0
	
	if (greatestValue < _L.bat.dpsTakenCalc) then greatestValue = _L.bat.dpsTakenCalc end
	
	if (includeDps and includeHps) then
		if (greatestValue < _L.bat.dpsDoneCalc) then greatestValue = _L.bat.dpsDoneCalc end
		if (greatestValue < _L.bat.hpsDoneCalc) then greatestValue = _L.bat.hpsDoneCalc end
		return greatestValue, (_L.bat.dpsTakenCalc or 0.0), (_L.bat.dpsDoneCalc or 0.0), (_L.bat.hpsDoneCalc or 0.0)
	elseif (includeHps) then
		if (greatestValue < _L.bat.dpsDoneCalc) then greatestValue = _L.bat.dpsDoneCalc end
		return greatestValue, (_L.bat.dpsTakenCalc or 0.0), (_L.bat.hpsDoneCalc or 0.0)
	end
	
	if (greatestValue < _L.bat.dpsDoneCalc) then greatestValue = _L.bat.dpsDoneCalc end
	return greatestValue, (_L.bat.dpsTakenCalc or 0.0), (_L.bat.dpsDoneCalc or 0.0)
end

function getUpdateFreq()
	return (_L.updateFreq or .5)
end

function getDynDPSformatStr(value, includeDps, includeHps)
	-- keep fall back settings in sync with getFormattedCurrentCombatData()
	local numberFormat = "\%.2f"
	if (includeDps == nil) then includeDps = true end
	if (includeHps == nil) then includeHps = false end
	
	if (value and (value > 100)) then
		numberFormat = "\%.0f"
	elseif (value and (value > 10)) then
		numberFormat = "\%.1f"
	end
	
	if (includeDps and includeHps) then
		return RED_FONT_COLOR_CODE..numberFormat..NORMAL_FONT_COLOR_CODE.." / "..GREEN_FONT_COLOR_CODE..numberFormat..NORMAL_FONT_COLOR_CODE.." / "..BLUE_LIGHT_FONT_COLOR_CODE..numberFormat
	elseif (includeHps) then
		return RED_FONT_COLOR_CODE..numberFormat..NORMAL_FONT_COLOR_CODE.." / "..BLUE_LIGHT_FONT_COLOR_CODE..numberFormat
	end
	
	return RED_FONT_COLOR_CODE..numberFormat..NORMAL_FONT_COLOR_CODE.." / "..GREEN_FONT_COLOR_CODE..numberFormat
end

function getCurrentDynDPSformatStr(value)
	return getDynDPSformatStr(value, 
		(CombatStats_Config.CombatStats_ShowDPSinMini), (CombatStats_Config.CombatStats_ShowHPSinMini) )
end

--
-- Support Functionality Funcs
--

function updateVisibility(hasTarget)
	if (hasTarget == nil) then
		if (TargetFrame:IsVisible()) then
			hasTarget = ( UnitCanAttack("player", "target") or UnitCanAttack("target", "player") )
		else
			hasTarget = 0
		end
	end
	if (CombatStats_Config.CombatStats_OnOff) then
		if (CombatStats_Config.CombatStats_HideOnNoTarget) then
			if (hasTarget == 1) then
				if (CombatStats_Config.CombatStats_Hide) then
					CombatStatsFrame:Hide()
				else
					CombatStatsFrame:Show()
					updateDynCombatTexts()	-- the data shown may not be current, so update it
				end
			else
				CombatStatsFrame:Hide()
			end
		else
			if (CombatStats_Config.CombatStats_Hide) then
				CombatStatsFrame:Hide()
			else
				CombatStatsFrame:Show()
				updateDynCombatTexts()	-- the data shown may not be current, so update it
			end
		end
	end
end


function showLastFightDPS()
	--
	--	Show the last fight DPS
	--
	local totalFightTime
	local playerDPS
	local petDPS
	local petPct
	local text = CS_LF_STATS_TEXT
	local playerN
	local data
	local i
	
	if( (_L.bInCombat == 0) and (_L.lastFight.finish > _L.lastFight.start)) then
		totalFightTime = _L.lastFight.finish - _L.lastFight.start
		petDPS = _L.lastFight.petDmg / totalFightTime
		playerDPS = _L.lastFight.playerDmg / totalFightTime
		
		if(lastFightPetDamge ~=0) then
			petPct =  (_L.lastFight.petDmg / _L.lastFight.playerDmg) * 100
		end
		text = text ..format(CS_LF_PLAYER_TEXT, totalFightTime,_L.lastFight.playerDmg,playerDPS)
		if(_L.lastFight.missDmg ~= 0) then
			text = text ..format(CS_LF_PLAYER_TEXT2, _L.lastFight.missDmg)
		end
		if(_L.lastFight.healed ~= 0) then
			text = text ..format(CS_LF_PLAYER_TEXT3, _L.lastFight.healed, (_L.lastFight.healed/totalFightTime) )
		end
		if(_L.lastFight.petDmg ~= 0) then
			text = text ..format(CS_LF_PET_TEXT, _L.lastFight.petDmg,petPct,petDPS)
		end
		msgPrint(text)
	end
end


function enterCombatMode(anEvent)
	debugPrint("enterCombatMode")
	--msgPrint("enterCombatMode - "..(anEvent or "nil").." event")
	
	if (anEvent) then _L.lastEnterEventCombatTime = GetTime()
	else _L.lastEnterDmgCombatTime = GetTime() end
	
	if (_L.bInCombat == 0) then
		_L.bInCombat = 1
		if ( not CombatStats_Config.CombatStats_OnOff ) then
			return
		end
		if ( CombatStats_Config.CombatStats_AutoBattleReset ) then
			reset()
		end
		
		_L.lastFight.start = GetTime()
		_L.lastFight.playerDmg = 0
		_L.lastFight.petDmg = 0
		_L.lastFight.missDmg = 0
		_L.lastFight.healed = 0
		
		_L.bat.givenCount = 0
		_L.bat.startGivenIdx = 1
		_L.bat.takenCount = 0
		_L.bat.startTakenIdx = 1
		_L.bat.givenHealCount = 0
		_L.bat.startHealGivenIdx = 1
		_L.bat.dpsTakenCalc = 0.0
		_L.bat.dpsDoneCalc = 0.0
		_L.bat.hpsDoneCalc = 0.0
		
		for i=1, table.getn(_L.bat.playersBatTots), 1 do
			_L.bat.playersBatTots[i] = {0,0,0,0,0,0}
			--_L.bat.playersBatTots[i] = { dmgEndTot = 0, dmgTakenEndTot = 0,
			--							dmgTot = 0, dmgTakenTot = 0 }
		end
		
		-- reset the update timer, but skip the first interations (to avoid displaying super high initial DPS values)
		_L.timeSinceLastUpdate = -(_L.updateFreq*1)
		
		updateVisibility()
	end
end


function endCombatMode(anEvent)
	debugPrint("endCombatMode")
	--msgPrint("endCombatMode - "..(anEvent or "nil").." event")
	
	if (_L.bInCombat == 1) then
		_L.bInCombat = 0
		if ( not CombatStats_Config.CombatStats_OnOff ) then 
			return
		end
		
		_L.lastFight.finish = GetTime()
		if ((_L.lastFight.playerDmg > 0) or (_L.lastFight.missDmg > 0) or (_L.lastFight.healed > 0)) then
			if(CombatStats_Config.CombatStats_EndOfFight) then
				showLastFightDPS()
			end
		end
		
		updateDynCombatInfo(_L.lastFight.finish)
		chkGarbage(_L.kGCafterEOFType)
		
		_L.lastEnterEventCombatTime = 0
		_L.lastEnterDmgCombatTime = 0
	end
	
	_L.petGUID = nil		-- reset in case the pet is changed
end


function chkIfCombatModeShouldEnd()
	-- So why is this being done?  Well when combat begins and ends is a little tricky
	-- WoW kindly notifies us when the player enter and leaves combat, BUT there
	-- currently is no way of knowing the order when events are triggered.  So it
	-- does occur that the first attack damage is missed because the enter combat notification
	-- event is recieved after the first combat log entry.  This was handled by simply
	-- tracking the in and out of combat state ourselves.  I idea being is if we are
	-- causing damaging we are in combat.  And along with the WoW combat events we where good.
	-- As WoW has evolved, blizzard seems to have a new idea.  You can damage but not be
	-- in combat, ie a bombing quest.  It does seem logical because you are doing damage
	-- that is way beyond what you could ever do with equipable items.
	--
	-- At this time CombatStats is going to continue to track all damage by a player
	-- as combat damage.  In the future this will likely be a setable option.
	
	debugPrint("chkIfCombatModeShouldEnd")
	
	if ((_L.bInCombat == 1) and (_L.lastEnterEventCombatTime == 0)) then
	
		-- so here is how the logic goes
		-- enterCombatMode is called whenever we do damage OR when WoW notifies of in combat
		-- enterCombatMode tracks when the last damage type enter or event enter combat calls it
		-- if we get a WoW enter combat we will not leave combat until we get a WoW leave combat
		-- notification but if we go into combat because of just damage then we leave combat
		-- state after 3 seconds of non-damage.
		--
		-- the check code is here to do what we want
		if ( (GetTime()-3) > _L.lastEnterDmgCombatTime) then
			endCombatMode()
		end
	end

end

function getSwingType(resisted, blocked, absorbed, critical, glancing, crushing)
	--debugPrint("res: "..(resisted or "nil")..",block: "..(blocked or "nil")..",abso: "..(absorbed or "nil")..",crit: "..(critical or "nil")..",glan: "..(glancing or "nil")..",crush: "..(crushing or "nil"))
	
	if (absorbed) then
		return "ABSORB"
	elseif (blocked) then
		return "BLOCK"
	elseif (resisted) then
		return "RESIST"
	elseif (crushing) then
		return "CRUSH"
	elseif (glancing) then
		return "GLANCE"
	elseif (critical) then
		return "CRIT"
	end
	
	return "ORDINARY"
end

function getInternalMissType(missType)
	local internalMissType = CombatStats.kMissType		-- default attack missing
	
	if (missType ~= nil) then
		if (missType == "PARRY") then internalMissType = CombatStats.kParryType
		elseif (missType == "EVADE") then internalMissType = CombatStats.kEvadeType
		elseif (missType == "DODGE") then internalMissType = CombatStats.kDodgeType
		elseif (missType == "DEFLECT") then internalMissType = CombatStats.kDeflectType
		elseif (missType == "BLOCK") then internalMissType = CombatStats.kBlockType
		elseif (missType == "RESIST") then internalMissType = CombatStats.kResistType
		elseif (missType == "IMMUNE") then internalMissType = CombatStats.kImmuneType
		elseif ((missType == "ABSORB") or (missType == "MISS")) then
			internalMissType = CombatStats.kMissType		-- not being recorded specially
		else debugPrint("CS ERR: MISS type known: "..missType)
		end
	end
	
	return internalMissType
end


function getInternalHitType(resisted, blocked, absorbed, critical, glancing, crushing)
	--debugPrint("res: "..(resisted or "nil")..",block: "..(blocked or "nil")..",abso: "..(absorbed or "nil")..",crit: "..(critical or "nil")..",glan: "..(glancing or "nil")..",crush: "..(crushing or "nil"))
	local hitType
	local missedType = ""
	
	if (blocked) then
		missedType = getInternalMissType("BLOCK")
	elseif (absorbed) then
		missedType = getInternalMissType("ABSORB")
	elseif (resisted) then
		missedType = getInternalMissType("RESIST")
	end
	
	if (critical) then
		hitType = CombatStats.kCritType
	else
		hitType = CombatStats.kHitType
	end
	
	return hitType, missedType
end


local function setDynCombatTexts(theString)
	debugPrint("setDynCombatTexts called with: "..theString)		-- DEBUG
	
	if(CombatStatsFrame:IsVisible()) then
		CombatStatsText:SetText(theString)
	end
	-- Update InfoBar if it is being used
	if(InfoBarFrame ~= nil) then
		if(CombatStatsIBFrame:IsVisible()) then
			CombatStatsIBFrameCenteredText:SetText(theString)
		end
	end
end


function updateDynCombatTexts()
	-- msgPrint("updateDynCombatTexts")			-- DEBUG
	local theString
	local greatestVal, takenCalc, CalcVal1, CalcVal2 = getFormattedCurrentCombatData()
	if (greatestVal > 10) then
		theString = format(_L.dpsForBg, takenCalc, CalcVal1, CalcVal2)
	elseif (greatestVal > 0) then
		theString = format(_L.dpsForSm, takenCalc, CalcVal1, CalcVal2)
	else
		theString = CS_MINIFRAME_DEFAULT_TEXT
	end
	
	--[[
	if (_L.bat.dpsDoneCalc ~= 0 or _L.bat.dpsTakenCalc ~= 0 or _L.bat.hpsDoneCalc ~= 0) then
		if (_L.bat.dpsDoneCalc > 10) or (_L.bat.hpsDoneCalc > 10) then
			--theString = format("|cffff2020\%.1f|cffffd200\ / |cff20ff20\%.1f", _L.bat.dpsTakenCalc, _L.bat.dpsDoneCalc)
			theString = format(_L.dpsForBg, _L.bat.dpsTakenCalc, _L.bat.dpsDoneCalc, _L.bat.hpsDoneCalc)
		else
			-- only a savings of about 0.005 KB
			--theString = format("|cffff2020\%.2f|cffffd200\ / |cff20ff20\%.2f", _L.bat.dpsTakenCalc, _L.bat.dpsDoneCalc)
			theString = format(_L.dpsForSm, _L.bat.dpsTakenCalc, _L.bat.dpsDoneCalc, _L.bat.hpsDoneCalc)
		end
	else
		theString = CS_MINIFRAME_DEFAULT_TEXT
	end
	]]
	
	-- Update CS main mini window
	if(CombatStatsFrame:IsVisible()) then
		CombatStatsText:SetText( theString )
	end
	-- Update InfoBar if it is being used
	if(InfoBarFrame ~= nil) then
		if(CombatStatsIBFrame:IsVisible()) then
			CombatStatsIBFrameCenteredText:SetText(theString)
		end
	end
	--[[
	-- cross check with blizzard to see if we are still really in battle
	if ((_L.bInCombat == 1) and (PlayerFrame ~= nil) and (not PlayerFrame.inCombat)) then
		endCombatMode()
	end
	]]
	
	-- ### KEEP TPCombatStatsTB_GetButtonText() function in sync with this function
end


function updateDynCombatInfo(curtime)
	local oldesttime = 0
	local oldesttimeTaken = 0
	local oldesttimeHeal = 0
	local deleteindex = 0
	local deleteindexTaken = 0
	local i
	local shouldUpdate = false
	
	--
	--	Offensive
	--
	for i = _L.bat.startGivenIdx, _L.bat.givenCount do
--	for k,v in pairs(battledmgGivenTBL) do
		-- Get rid of old dps entries and adjust the totals
		if ( (curtime - CombatStats_DPSLen) > _L.bat.timeStampGiven[i] ) then
			--msgPrint("Offensive subtract: "..i)
			--msgPrint("_L.bat.playerGiven[i]: ".._L.bat.playerGiven[i])
			
			-- Subtract this entry from the totals
			_L.bat.playersBatTots[_L.bat.playerGiven[i]][kDmgEndTotIdx] = 
				_L.bat.playersBatTots[_L.bat.playerGiven[i]][kDmgEndTotIdx]
				- _L.bat.dmgGiven[i]
		--	_L.bat.playersBatTots[v.playerN][kDmgEndTotIdx] = _L.bat.playersBatTots[v.playerN][kDmgEndTotIdx] - v.dmgP
			-- Mark for later removal *We can't do the remove here cause table.remove reindexes the list and bones our loop
			--deleteindex = k
			
			_L.bat.startGivenIdx = i + 1
		else
			-- were into the good stuff now, stop
			--oldesttime = v.timestamp
			oldesttime = _L.bat.timeStampGiven[i]
			break
		end
	end
	--[[ maybe re-impliment back entry cleanup code.  It this time, it is not an issue because there are not that many entries ]]
		
	
	--
	--	Defensive
	--
	for i = _L.bat.startTakenIdx, _L.bat.takenCount do
	--for w,y in pairs(battledmgTakenTBL) do
		-- adjust the totals
		if ( (curtime - CombatStats_DPSLen) > _L.bat.timeStampTaken[i] ) then
			-- Subtract this entry from the totals
			_L.bat.playersBatTots[_L.bat.playerTaken[i]][kDmgTakenEndTotIdx] = 
				_L.bat.playersBatTots[_L.bat.playerTaken[i]][kDmgTakenEndTotIdx] - _L.bat.dmgTaken[i]
			_L.bat.startTakenIdx = i + 1
		else
			-- were into the good stuff now, stop
			--oldesttimeTaken = y.timestamp
			oldesttimeTaken = _L.bat.timeStampTaken[i]
			break
		end
	end
	--[[ maybe re-impliment back entry cleanup code.  It this time, it is not an issue because there are not that many entries ]]
	
	--
	--	Heals
	--
	for i = _L.bat.startHealGivenIdx, _L.bat.givenHealCount do
		if ( (curtime - CombatStats_DPSLen) > _L.bat.timeStampHeal[i] ) then
			-- Subtract this entry from the totals
			_L.bat.playersBatTots[_L.bat.playerHeal[i]][kHealsEndTotIdx] = 
				_L.bat.playersBatTots[_L.bat.playerHeal[i]][kHealsEndTotIdx] - _L.bat.HealGiven[i]
			_L.bat.startHealGivenIdx = i + 1
		else
			oldesttimeHeal = _L.bat.timeStampHeal[i]
			break
		end
	end
	--[[ maybe re-impliment back entry cleanup code.  It this time, it is not an issue because there are not that many entries ]]


	-- Update player DPS
	if ((_L.bat.playersBatTots[_L.bat.yourIdx][kDmgEndTotIdx] ~= nil ) and (oldesttime ~= 0)) then
		_L.bat.dpsDoneCalc = (_L.bat.playersBatTots[_L.bat.yourIdx][kDmgEndTotIdx] / (curtime - oldesttime) )
		shouldUpdate = true
	--else
	--	_L.bat.dpsDoneCalc = 0.0
	end
	if ((_L.bat.playersBatTots[_L.bat.yourIdx][kDmgTakenEndTotIdx] ~= nil) and (oldesttimeTaken ~= 0)) then
		_L.bat.dpsTakenCalc = (_L.bat.playersBatTots[_L.bat.yourIdx][kDmgTakenEndTotIdx] / (curtime - oldesttimeTaken) )
		shouldUpdate = true
	--else
	--	_L.bat.dpsTakenCalc = 0.0
	end
	if ((_L.bat.playersBatTots[_L.bat.yourIdx][kHealsEndTotIdx] ~= nil) and (oldesttimeHeal ~= 0)) then
		_L.bat.hpsDoneCalc = (_L.bat.playersBatTots[_L.bat.yourIdx][kHealsEndTotIdx] / (curtime - oldesttimeHeal) )
		shouldUpdate = true
	--else
	--	_L.bat.hpsDoneCalc = 0.0
	end
	
	-- update all the small dmgTakenPS,DPS,HPS displays
	if (shouldUpdate) then updateDynCombatTexts() end

	--
	--	If the data frame is visible
	--	update the selected attack "Real time"
	--
	if (CombatStatsDataFrame:IsVisible()) then
		if( CombatStatsAttackDropDown_GetText() ~= CS_DROPDOWN_SELECT_TEXT) then
			CombatStats_UpdateDetails( CombatStatsAttackDropDown_GetText())
		end
	end
	
end


function chkGarbage(typeToChk)
	--msgPrint("chkGarbage")
	typeToChk = typeToChk or 0
	
	if (CombatStats_Config.CombatStats_GC == 0) then
		return
	elseif (CombatStats_Config.CombatStats_GC == _L.kGCafterEOFType) then
		if (typeToChk == _L.kGCafterEOFType) then
			collectgarbage('collect')
			--msgPrint("chkGarbage (A) ---- collect")
		end
		return
	elseif (CombatStats_Config.CombatStats_GC == _L.kGCatRestType) then
		if ((typeToChk == _L.kGCatRestType) and (IsResting())) then
			collectgarbage('collect')
			--msgPrint("chkGarbage (B)---- collect")
		end
		return
	end
	
	-- someday maybe use PLAYER_FLAGS_CHANGED
end



function getAddonVersionNumber()
	local addonVersionNum = string.gsub(format("%.2f",tonumber(_L.VERSION or "1.00")), "([^%a^%d])", "")
	return tonumber(addonVersionNum)
end
_L.VERSNUM = getAddonVersionNumber()		-- set the variable, now that the function exists


function chatCommandUsage(whichone)
	whichone = whichone or 1
	
	msgPrint(CS_CMD_HELP_OLD_TEXT)
	msgPrint(CS_CMD_ENABLE_OLD_TEXT)
	msgPrint(CS_CMD_TARGET_OLD_TEXT)
	msgPrint(CS_CMD_MOUSEOVER_OLD_TEXT)
	msgPrint(CS_CMD_ENDFLIGHT_OLD_TEXT)
	msgPrint(CS_CMD_HIDE_OLD_TEXT)
	msgPrint(CS_CMD_SHOWDPSMINI_OLD_TEXT)
	msgPrint(CS_CMD_SHOWHPSMINI_OLD_TEXT)
	msgPrint(CS_CMD_TOOLTIP_OLD_TEXT)
	
	msgPrint(CS_CMD_SHOWSTAT_TEXT)
	
	msgPrint(CS_CMD_RESET_OLD_TEXT)
	msgPrint(CS_CMD_RESETWIN_TEXT)
	msgPrint(CS_CMD_SETMAXUPS_TEXT)
end


--
--	Damage Done and Taken Funcs
--

	--	Add damage done to the DPS table
function addDPSEntry(timeStamp, p, d)
	local playerIdx = 0
	
	-- sanity check
	if (d == nil) then return end
	
	-- get player index
	for i=1, table.getn(_L.bat.playersTBL), 1 do
		if(_L.bat.playersTBL[i] == p) then
			playerIdx = i
			break
		end
	end
	-- add player to table if it does not exist
	if (playerIdx == 0) then
		-- add player index
		table.insert(_L.bat.playersTBL, p)
		playerIdx = table.getn(_L.bat.playersTBL)
	end
	
	if (playerIdx == 0) then msgPrint("ERR: playerIdx not set") return end
	
	_L.bat.givenCount = _L.bat.givenCount + 1
	_L.bat.timeStampGiven[_L.bat.givenCount] = timeStamp
	_L.bat.playerGiven[_L.bat.givenCount] = playerIdx
	_L.bat.dmgGiven[_L.bat.givenCount] = d

	-- add data totals if this is a new player entry
	if (_L.bat.playersBatTots[playerIdx] == nil) then
		--msgPrint("new player index: "..playerIdx..", for "..p)
		_L.bat.playersBatTots[playerIdx] = {0,0,0,0,0,0}
		--_L.bat.playersBatTots[playerIdx] = { dmgEndTot = 0, dmgTakenEndTot = 0,
		--						dmgTot = 0, dmgTakenTot = 0 }
	end
	
	-- Keep running totals on group members to shorten calculation times
	-- We'll subtract off the expired entries during OnUpdate
	if (_L.bat.playersBatTots[playerIdx][kDmgEndTotIdx] == nil) then
		--msgPrint("_L.bat.playersBatTots[playerIdx][kDmgEndTotIdx] == nil")
		_L.bat.playersBatTots[playerIdx][kDmgEndTotIdx] = d
		_L.bat.playersBatTots[playerIdx][kDmgTotIdx] = d
	else
		--msgPrint("_L.bat.playersBatTots[playerIdx][kDmgEndTotIdx] == " .._L.bat.playersBatTots[playerIdx][kDmgEndTotIdx] )
		_L.bat.playersBatTots[playerIdx][kDmgEndTotIdx] = _L.bat.playersBatTots[playerIdx][kDmgEndTotIdx] + d
		_L.bat.playersBatTots[playerIdx][kDmgTotIdx] = _L.bat.playersBatTots[playerIdx][kDmgTotIdx] + d
		--msgPrint("AFTER : _L.bat.playersBatTots[playerIdx][kDmgEndTotIdx] == " .._L.bat.playersBatTots[playerIdx][kDmgEndTotIdx] )
	end
	--msgPrint("Total : " .._L.bat.playersBatTots[playerIdx][kDmgEndTotIdx])
end


	--	Add damage taken to the DPS table
function addDefDPSEntry(timeStamp, p, d)
	local playerIdx = 0
	
	-- sanity check
	if (d == nil) then return end
	
	--msgPrint("Adding Entry[" .. p .. "] = " .. d);	-- debug
	
	-- get player index
	for i=1, table.getn(_L.bat.playersTBL), 1 do
		if(_L.bat.playersTBL[i] == p) then
			playerIdx = i
			break
		end
	end
	-- add player to table if it does not exist
	if (playerIdx == 0) then
		-- add player index
		table.insert(_L.bat.playersTBL, p)
		playerIdx = table.getn(_L.bat.playersTBL)
	end
	
	if (playerIdx == 0) then msgPrint("ERR: playerIdx not set") end
	
	_L.bat.takenCount = _L.bat.takenCount + 1
	_L.bat.timeStampTaken[_L.bat.takenCount] = timeStamp
	_L.bat.playerTaken[_L.bat.takenCount] = playerIdx
	_L.bat.dmgTaken[_L.bat.takenCount] = d
	
	-- add data totals if this is a new player entry
	if (_L.bat.playersBatTots[playerIdx] == nil) then
		_L.bat.playersBatTots[playerIdx] = {0,0,0,0,0,0}
		--_L.bat.playersBatTots[playerIdx] = { dmgEndTot = 0, dmgTakenEndTot = 0,
		--						dmgTot = 0, dmgTakenTot = 0 }
	end
	
	-- Keep running totals on group members to shorten calculation times
	-- We'll subtract off the expired entries during OnUpdate
	if (_L.bat.playersBatTots[playerIdx][kDmgTakenEndTotIdx] == nil) then
		--msgPrint("_L.bat.playersBatTots[playerIdx][kDmgEndTotIdx] == nil")
		_L.bat.playersBatTots[playerIdx][kDmgTakenEndTotIdx] = d
		_L.bat.playersBatTots[playerIdx][kDmgTakenTotIdx] = d
	else
		--msgPrint("_L.bat.playersBatTots[playerIdx][kDmgEndTotIdx] == " .._L.bat.playersBatTots[playerIdx][kDmgEndTotIdx] )
		_L.bat.playersBatTots[playerIdx][kDmgTakenEndTotIdx] = _L.bat.playersBatTots[playerIdx][kDmgTakenEndTotIdx] + d
		_L.bat.playersBatTots[playerIdx][kDmgTakenTotIdx] = _L.bat.playersBatTots[playerIdx][kDmgTakenTotIdx] + d
		--msgPrint("AFTER : _L.bat.playersBatTots[playerIdx][kDmgEndTotIdx] == " .._L.bat.playersBatTots[playerIdx][kDmgEndTotIdx] )
	end
	--msgPrint("Total : " .._L.bat.playersBatTots[playerIdx][kDmgEndTotIdx])
end


	--	Add heals given to table
function addHealEntry(timeStamp, p, h)
	local playerIdx = 0
	
	-- sanity check
	if (h == nil) then return end
	
	--msgPrint("Adding Heal Entry[" .. p .. "] = " .. h);	-- debug
	
	-- get player index
	for i=1, table.getn(_L.bat.playersTBL), 1 do
		if(_L.bat.playersTBL[i] == p) then
			playerIdx = i
			break
		end
	end
	-- add player to table if it does not exist
	if (playerIdx == 0) then
		-- add player index
		table.insert(_L.bat.playersTBL, p)
		playerIdx = table.getn(_L.bat.playersTBL)
	end
	
	if (playerIdx == 0) then msgPrint("ERR: playerIdx not set") end
	
	_L.bat.givenHealCount = _L.bat.givenHealCount + 1
	_L.bat.timeStampHeal[_L.bat.givenHealCount] = timeStamp
	_L.bat.playerHeal[_L.bat.givenHealCount] = playerIdx
	_L.bat.HealGiven[_L.bat.givenHealCount] = h
	
	-- add data totals if this is a new player entry
	if (_L.bat.playersBatTots[playerIdx] == nil) then
		_L.bat.playersBatTots[playerIdx] = {0,0,0,0,0,0}
		--_L.bat.playersBatTots[playerIdx] = { dmgEndTot = 0, dmgTakenEndTot = 0,
		--						dmgTot = 0, dmgTakenTot = 0 }
	end
	
	-- Keep running totals on group members to shorten calculation times
	-- We'll subtract off the expired entries during OnUpdate
	if (_L.bat.playersBatTots[playerIdx][kHealsEndTotIdx] == nil) then
		_L.bat.playersBatTots[playerIdx][kHealsEndTotIdx] = h
		_L.bat.playersBatTots[playerIdx][kHealsTotIdx] = h
	else
		_L.bat.playersBatTots[playerIdx][kHealsEndTotIdx] = _L.bat.playersBatTots[playerIdx][kHealsEndTotIdx] + h
		_L.bat.playersBatTots[playerIdx][kHealsTotIdx] = _L.bat.playersBatTots[playerIdx][kHealsTotIdx] + h
	end
end


function addSpecialData(specialName,type,dmg,sysdatetime,heal,isPet)
	--msgPrint("Adding : "..specialName .." with damage : "..(dmg or "nil")..", at : "..sysdatetime)
	if (dmg == nil) then return end
	heal = heal or false
	isPet = isPet or false
	local	tmpType
	
	-- make sure the dmg variable is a number 
	--		- already stored variables have been corrected in the load verify function
	dmg = tonumber(dmg)
	
	if(specialName ~= CS_ATTACK_DEFENSE_LABEL and not heal ) then
		--_L.lastFight.playerDmg = _L.lastFight.playerDmg + dmg		-- moved out 2008.09.06
		--if (isPet) then _L.lastFight.petDmg = _L.lastFight.petDmg + dmg end		-- moved out 2008.09.06
	end
		
	if (type == CombatStats.kDotType) then
		specialName = specialName..CS_ATTACK_DOT_SUF_LABEL
	elseif ( type == CombatStats.kHotType ) then
		specialName = specialName..CS_ATTACK_HOT_SUF_LABEL
	end
	
	--
	--	If we haven't logged this attack
	--	Create a record in the table for it
	--
	-- TOTAL HACK HERE ---- FIX
	if (CombatStats.AttackHistDB.GetAttackRec(CS_ATTACK_TOTAL_LABEL,false) == nil) then
		CombatStats.AttackHistDB.GetAttackRec(CS_ATTACK_TOTAL_LABEL,true,true)
	end
	
	if (CombatStats.AttackHistDB.GetAttackRec(specialName,false) == nil) then
		CombatStats.AttackHistDB.GetAttackRec(specialName,true,true)
		CombatStats.DetailFrame.SetDirtyAttackMenu()
	end
	
	--
	--	Now add the data
	--

	incrementSpecialDataRecord(specialName,type,dmg,sysdatetime,heal)
	
	-- increment the totals
			--	Don't add (Defensive info) to overall stats
	if (specialName ~= CS_ATTACK_DEFENSE_LABEL) then
		if (heal) then
			--	Not ideal but the total HOT heal is being included in the non-crit total record area -- should fix someday
			if (type == CombatStats.kHotType) then tmpType = CombatStats.kHitType else tmpType = type end
			
			incrementSpecialDataRecord(CS_ATTACK_HEAL_TOT_LABEL,tmpType,dmg,sysdatetime,heal,isPet)
			-- update detailed window if this is the selected attack
			CombatStats_UpdateDetails(CS_ATTACK_HEAL_TOT_LABEL, true)
		else
			--	Not ideal but the total DOT damage is being included in the non-crit total record area -- should fix someday
			if (type == CombatStats.kDotType) then tmpType = CombatStats.kHitType else tmpType = type end
			
			incrementSpecialDataRecord(CS_ATTACK_TOTAL_LABEL,tmpType,dmg,sysdatetime,heal,isPet)
			-- update detailed window if this is the selected attack
			CombatStats_UpdateDetails(CS_ATTACK_TOTAL_LABEL, true)
		end
	end
	
	-- update detailed window if this is the selected attack
	CombatStats_UpdateDetails(specialName, true)
	
end


function addSpecialDataMiss(specialName,type,dmg,sysdatetime,heal,isPet,missDmg,perOfdmg)
	--msgPrint("Adding (miss): "..specialName .." with damage : "..(dmg or "nil").." with missdam : "..(missDmg or "nil")..", at : "..sysdatetime)
	if ( not ( (type == CombatStats.kMissType) or
			(type == CombatStats.kDodgeType) or
			(type == CombatStats.kParryType) or
			(type == CombatStats.kBlockType) or
			(type == CombatStats.kEvadeType) or
			(type == CombatStats.kResistType) or
			(type == CombatStats.kImmuneType) or
			(type == CombatStats.kDeflectType) )) then return end
	
	dmg = dmg or 0
	missDmg = missDmg or 0
	perOfdmg = perOfdmg or 0
	heal = heal or false
	isPet = isPet or false
	-- make sure the dmg variable is a number 
	dmg = tonumber(dmg)	
	--
	--	If we haven't logged this attack
	--	Create a record in the table for it
	-- TOTAL HACK HERE ---- FIX
	if (CombatStats.AttackHistDB.GetAttackRec(specialName,false) == nil) then
		CombatStats.AttackHistDB.GetAttackRec(specialName,true,true)
		CombatStats.DetailFrame.SetDirtyAttackMenu()
	end
	
	--
	--	Now add the miss data
	--
	CombatStats.AttackHistDB.AddMissAttackData(specialName, type, missDmg, perOfdmg)
	
	local specAttck = CombatStats.AttackHistDB.GetAttackRec(specialName,false)
	if (specAttck == nil) then
		msgPrint("GetAttackRec returned nil for "..specialName)
	else
		specAttck.totalSwings = specAttck.totalSwings + 1
		specAttck.lastUpdate = sysdatetime
	end

	-- update detailed window if this is the selected attack
	CombatStats_UpdateDetails(specialName, true)
end


function incrementSpecialDataRecord(recordName,type,dmg,sysdatetime,heal)
--	local specAttck = specialAttacks[recordName]
	local specAttck = CombatStats.AttackHistDB.GetAttackRec(recordName,true,true)		-- ok to be true?? 2008.08
	
	if (specAttck == nil) then msgPrint("GetAttackRec returned nil for "..recordName); return; end
	
	-- make sure the dmg variable is a number - already stored variable have been corrected in the load verify function
	dmg = tonumber(dmg)
	
	--
	--	Now update the records data if healing flag is not set
	--
	
--	if (not heal) then
	specAttck.totalSwings = specAttck.totalSwings + 1
	specAttck.lastUpdate = sysdatetime
	--_L.dirtyAttackMenu = 1
	CombatStats.DetailFrame.SetDirtyAttackMenu()
	if(type == CombatStats.kCritType) then
		specAttck.isDOT = 0
		specAttck.totalCrits = specAttck.totalCrits +1
		specAttck.totalCritDmg = specAttck.totalCritDmg + dmg
		specAttck.totalHits = specAttck.totalHits + 1
		specAttck.lastCrit = sysdatetime
		--
		--	Check to see if this dmg
		--	is a new min or max
		--
		if((dmg < specAttck.minCrit) or (specAttck.minCrit == 0)) then
			specAttck.minCrit = dmg
			specAttck.minCritTime = sysdatetime
		end
		if(dmg > specAttck.maxCrit) then
			specAttck.maxCrit = dmg
			specAttck.maxCritTime = sysdatetime
		end
	elseif(type == CombatStats.kHitType) then
		specAttck.isDOT = 0
		specAttck.totalNonCrits = specAttck.totalNonCrits +1
		specAttck.totalRegDmg = specAttck.totalRegDmg + dmg
		specAttck.totalHits = specAttck.totalHits + 1
		--
		--	Check to see if this dmg
		--	is a new min or max
		--
		if((dmg < specAttck.minReg) or (specAttck.minReg == 0)) then
			specAttck.minReg = dmg
			specAttck.minRegTime = sysdatetime
		end
		if(dmg > specAttck.maxReg) then
			specAttck.maxReg = dmg
			specAttck.maxRegTime = sysdatetime
		end
	elseif(type == CombatStats.kDotType) then
		specAttck.isDOT = 1
		specAttck.dotTicks = specAttck.dotTicks +1
		specAttck.dotDmg = specAttck.dotDmg + dmg
		specAttck.totalHits = specAttck.totalHits + 1
		--
		--	Check to see if this dmg
		--	is a new min or max
		--
		if((dmg < specAttck.minReg) or (specAttck.minReg == 0)) then
			specAttck.minReg = dmg
			specAttck.minRegTime = sysdatetime
		end
		if(dmg > specAttck.maxReg) then
			specAttck.maxReg = dmg
			specAttck.maxRegTime = sysdatetime
		end
	elseif(type == CombatStats.kHotType) then
		specAttck.isDOT = 2
		specAttck.dotTicks = specAttck.dotTicks +1
		specAttck.dotDmg = specAttck.dotDmg + dmg
		specAttck.totalHits = specAttck.totalHits + 1
		--
		--	Check to see if this dmg
		--	is a new min or max
		--
		if((dmg < specAttck.minReg) or (specAttck.minReg == 0)) then
			specAttck.minReg = dmg
			specAttck.minRegTime = sysdatetime
		end
		if(dmg > specAttck.maxReg) then
			specAttck.maxReg = dmg
			specAttck.maxRegTime = sysdatetime
		end
	elseif((type == CombatStats.kMissType) or
			(type == CombatStats.kDodgeType) or
			(type == CombatStats.kParryType) or
			(type == CombatStats.kBlockType) or
			(type == CombatStats.kEvadeType) or
			(type == CombatStats.kResistType) or
			(type == CombatStats.kImmuneType) or 
			(type == CombatStats.kDeflectType)) then
		msgPrint("Err: incrementSpecialDataRecord func should no longer be used to handle miss types")
		addSpecialDataMiss(recordName,type,0,sysdatetime,heal,isPet,0,1)
		addSpecialDataMiss(CS_ATTACK_TOTAL_LABEL,type,0,sysdatetime,heal,isPet,0,1)
	end
		
--	end
end


	-- strange no one is calling this function - 2008.04 ??
function subtractFromSpecialDataTotals(attackRecord)
	-- passing in the attackRecord a slower operation than individual args 
	-- so this function should be a low use call
	
	--- FIX!!!!! to behave correctly for heals - 08.2008
	
	--local specAttck = specialAttacks[CS_ATTACK_TOTAL_LABEL]
	local specAttck = CombatStats.AttackHistDB.GetAttackRec(CS_ATTACK_TOTAL_LABEL,false)
	
	specAttck.dotDmg = specAttck.dotDmg - attackRecord.dotDmg
	specAttck.dotTicks = specAttck.dotTicks - attackRecord.dotTicks
	specAttck.totalSwings = specAttck.totalSwings - attackRecord.totalSwings
	
	-- FIX
	--[[
	specAttck.totalMisses = specAttck.totalMisses - attackRecord.totalMisses
	specAttck.totalDodged = specAttck.totalDodged - attackRecord.totalDodged
	specAttck.totalParried = specAttck.totalParried - attackRecord.totalParried
	specAttck.totalEvaded = specAttck.totalEvaded - attackRecord.totalEvaded
	specAttck.totalBlocked = specAttck.totalBlocked - attackRecord.totalBlocked
	specAttck.totalResisted = specAttck.totalResisted - attackRecord.totalResisted
	specAttck.totalImmuned = specAttck.totalImmuned - attackRecord.totalImmuned
	specAttck.totalDeflected = specAttck.totalDeflected - attackRecord.totalDeflected
	]]
	-- Note xxxxDmgPercent 's do not need to be changed
	specAttck.totalHits = specAttck.totalHits - attackRecord.totalHits
	specAttck.totalNonCrits = specAttck.totalNonCrits - attackRecord.totalNonCrits
	specAttck.totalCrits = specAttck.totalCrits - attackRecord.totalCrits
	specAttck.totalRegDmg = specAttck.totalRegDmg - attackRecord.totalRegDmg
	specAttck.totalCritDmg = specAttck.totalCritDmg - attackRecord.totalCritDmg
end

--
--	Settings Accessors Funcs
--

function setEnable(theValue)
	-- nil theValue is for toggling
	theValue = isTrueValue(theValue)
	
	if (theValue ~= nil) then
		if (CombatStats_Config.CombatStats_OnOff == theValue) then return end
		CombatStats_Config.CombatStats_OnOff = theValue
	else
		if (CombatStats_Config.CombatStats_OnOff)
			then CombatStats_Config.CombatStats_OnOff = false
			else CombatStats_Config.CombatStats_OnOff = true
		end
	end

	-- value has been changed
	if (CombatStats_Config.CombatStats_OnOff == 0) then
		setDynCombatTexts(CS_MINIFRAME_DISABLE_TEXT)
		--CombatStats_SetCombatText(CS_MINIFRAME_DISABLE_TEXT,1)
		--setDynCombatTexts(CS_MINIFRAME_DISABLE_TEXT)
		--CombatStatsFrame:Hide()
	else
		updateDynCombatTexts()
		--setDynCombatTexts(CS_MINIFRAME_DEFAULT_TEXT)
		--CombatStats_SetCombatText(nil,2)
	end
--	updateDynCombatTexts()
	updateWhenActiveRegistery(CombatStatsFrame)
	updateVisibility()
	return(CombatStats_Config.CombatStats_OnOff)
end

function setTarget(theValue)
	-- nil theValue is for toggling
	theValue = isTrueValue(theValue)
	if (theValue ~= nil) then
		CombatStats_Config.CombatStats_HideOnNoTarget = theValue
	else
		if (CombatStats_Config.CombatStats_HideOnNoTarget) 
			then CombatStats_Config.CombatStats_HideOnNoTarget = false
			else CombatStats_Config.CombatStats_HideOnNoTarget = true
		end
	end
	
	updateVisibility()
	return(CombatStats_Config.CombatStats_HideOnNoTarget)
end

function setMouseOver(theValue)
	-- nil theValue is for toggling
	theValue = isTrueValue(theValue)
	
	if (theValue ~= nil) then
		CombatStats_Config.CombatStats_UseMouseOver = theValue
	else
		if (CombatStats_Config.CombatStats_UseMouseOver) 
			then CombatStats_Config.CombatStats_UseMouseOver = false
			else CombatStats_Config.CombatStats_UseMouseOver = true
		end
	end
	
	updateVisibility()
	return(CombatStats_Config.CombatStats_UseMouseOver)
end

function setEndFight(theValue)
	-- nil theValue is for toggling
	theValue = isTrueValue(theValue)
	
	if (theValue ~= nil) then
		CombatStats_Config.CombatStats_EndOfFight = theValue
	else
		if (CombatStats_Config.CombatStats_EndOfFight) 
			then CombatStats_Config.CombatStats_EndOfFight = false
			else CombatStats_Config.CombatStats_EndOfFight = true
		end
	end
	
	updateVisibility()
	return(CombatStats_Config.CombatStats_EndOfFight)
end

function setHide(theValue)
	-- nil theValue is for toggling
	theValue = isTrueValue(theValue)
	
	if (theValue ~= nil) then
		CombatStats_Config.CombatStats_Hide = theValue
	else
		if (CombatStats_Config.CombatStats_Hide) 
			then CombatStats_Config.CombatStats_Hide = false
			else CombatStats_Config.CombatStats_Hide = true
		end
	end
	
	updateVisibility()
	return(CombatStats_Config.CombatStats_Hide)
end

function setShowTtip(theValue)
	-- nil theValue is for toggling
	theValue = isTrueValue(theValue)
	
	if (theValue ~= nil) then
		CombatStats_Config.CombatStats_ShowTtip = theValue
	else
		if (CombatStats_Config.CombatStats_ShowTtip) 
			then CombatStats_Config.CombatStats_ShowTtip = false
			else CombatStats_Config.CombatStats_ShowTtip = true
		end
	end
	
	updateVisibility()
	return(CombatStats_Config.CombatStats_ShowTtip)
end

function setShowDPSinMini(theValue)
	-- nil theValue is for toggling
	theValue = isTrueValue(theValue)

	if (theValue ~= nil) then
		CombatStats_Config.CombatStats_ShowDPSinMini = theValue
	else
		if (CombatStats_Config.CombatStats_ShowDPSinMini) 
			then CombatStats_Config.CombatStats_ShowDPSinMini = false
			else CombatStats_Config.CombatStats_ShowDPSinMini = true
		end
	end
	
	resetMiniDPSFormat()
	updateVisibility()
	return(CombatStats_Config.CombatStats_ShowDPSinMini)
end

function setShowHPSinMini(theValue)
	-- nil theValue is for toggling
	theValue = isTrueValue(theValue)
	
	if (theValue ~= nil) then
		CombatStats_Config.CombatStats_ShowHPSinMini = theValue
	else
		if (CombatStats_Config.CombatStats_ShowHPSinMini) 
			then CombatStats_Config.CombatStats_ShowHPSinMini = false
			else CombatStats_Config.CombatStats_ShowHPSinMini = true
		end
	end
	
	resetMiniDPSFormat()
	updateVisibility()
	return(CombatStats_Config.CombatStats_ShowHPSinMini)
end

function setAutoBatReset(theValue)
	-- nil theValue is for toggling
	theValue = isTrueValue(theValue)
	
	if (theValue ~= nil) then
		CombatStats_Config.CombatStats_AutoBattleReset = theValue
	else
		if (CombatStats_Config.CombatStats_AutoBattleReset) 
			then CombatStats_Config.CombatStats_AutoBattleReset = false
			else CombatStats_Config.CombatStats_AutoBattleReset = true
		end
	end
	
	return(CombatStats_Config.CombatStats_AutoBattleReset)
end

function updateFramePer(theValue)
	--msgPrint("updateFramePer")
	theValue = tonumber(theValue) or 10
	
	if ((theValue == 0) or (theValue < .001)) then
		theValue = .001
	elseif (theValue > 99) then
		theValue = 99
	end
	
	CombatStats_Config.CombatStats_MaxUpPS = theValue
	return(CombatStats_Config.CombatStats_MaxUpPS)
end

function setMaxUPS()
	StaticPopup_Show("CombatStats_GUI_Get_Max_UpdatePreSec");
end

--
-- Utility Funcs
--

function arrTBLItemCount(theTBL)
	-- unknown why table.getn only works on aTable[2] and not aTable["fred"]
	local i
	local aCount=0
	for _,_ in pairs(theTBL) do aCount=aCount+1 end
	return aCount
end


function dumpInternalStats()
	--local num = GetNumAddOns()
	local mem = -1
	local cpu = -1
	local count
	local info1, info2
	local text
	--local _, _, _, khaosEnabled, _, _, _ = GetAddOnInfo("Khaos")
	--local _, _, _, titanEnabled, _, _, _ = GetAddOnInfo("Titan")
	--local _, _, _, infobarEnabled, _, _, _ = GetAddOnInfo("InfoBar")
	
	msgPrint("CombatStats")
	
	-- get memory & CPU usage
	UpdateAddOnMemoryUsage()
	mem = GetAddOnMemoryUsage(kAddonName)
	cpu = GetAddOnCPUUsage(kAddonName)
	--[[
	for i = 1, num do
		if (kAddonName == GetAddOnInfo(i)) then 
			mem = GetAddOnMemoryUsage(i)
			cpu = GetAddOnCPUUsage(i)
			break
		end
	end	]]
	msgPrint(" Memory used: "..format("%.1f", mem).."(KB), CPU usage: "..cpu)
	
	msgPrint(" Outside addon integration")
	if (_L.haveKhaos) then msgPrint("   Khaos integration is active")
	else msgPrint("   Khaos is not installed or not active") end
	if (_L.haveInfoBar) then msgPrint("   InfoBar integration is active")
	else msgPrint("   InfoBar is not installed or not active") end

	-- storage counts
	msgPrint(" CombatStats_Config item count: "..arrTBLItemCount(CombatStats_Config))
	msgPrint(" _L.bat.playersBatTots item count: "..arrTBLItemCount(_L.bat.playersBatTots))
	msgPrint(" gCS item count: "..arrTBLItemCount(gCS))
	msgPrint(" _L item count: "..arrTBLItemCount(_L))
--	msgPrint(" attackNames idx count: "..arrTBLItemCount(attackNames))
--	msgPrint(" groupNames idx count: "..arrTBLItemCount(groupNames))
	msgPrint(" specialAttacks item count: "..arrTBLItemCount(specialAttacks))
--	msgPrint(" specialAttackLog item count: "..arrTBLItemCount(specialAttackLog))

end


function dumpConfigVariables()
	
	msgPrint("CombatStats_Config data:")

	msgPrint(" CombatStats_OnOff: "..(CombatStats_Config.CombatStats_OnOff or "nil"))
	msgPrint(" CombatStats_HideOnNoTarget: "..(CombatStats_Config.CombatStats_HideOnNoTarget or "nil"))
	msgPrint(" CombatStats_EndOfFight: "..(CombatStats_Config.CombatStats_EndOfFight or "nil"))
	msgPrint(" CombatStats_UseMouseOver: "..(CombatStats_Config.CombatStats_UseMouseOver or "nil"))
	msgPrint(" CombatStats_Hide: "..(CombatStats_Config.CombatStats_Hide or "nil"))
	msgPrint(" CombatStats_ShowTtip: "..(CombatStats_Config.CombatStats_ShowTtip or "nil"))
	msgPrint(" CombatStats_ShowDPSinMini: "..(CombatStats_Config.CombatStats_ShowDPSinMini or "nil"))
	msgPrint(" CombatStats_ShowHPSinMini: "..(CombatStats_Config.CombatStats_ShowHPSinMini or "nil"))
	msgPrint(" CombatStats_AttackMenuSort: "..(CombatStats_Config.CombatStats_AttackMenuSort or "nil"))
	msgPrint(" CombatStats_AttackMenu1stGrping: "..(CombatStats_Config.CombatStats_AttackMenu1stGrping or "nil"))
	msgPrint(" CombatStats_AutoBattleReset: "..(CombatStats_Config.CombatStats_AutoBattleReset or "nil"))
	msgPrint(" CombatStats_GC: "..(CombatStats_Config.CombatStats_GC or "nil"))
end


function dumpArrayString(theArray, thePreLabel)
	local aString = ""
	thePreLabel = thePreLabel or "idx"
	
	--for i=1, table.getn(theArray) do
	for i,j in pairs(theArray) do
		aString = aString..thePreLabel.."["..i.."] = "..theArray[i]..", "
	end
	
	return aString
end


--
--	InfoBar Funcs
--

function combatStatsIB_OnLoad(self)
	self.info = 
	{
		name=_L.TITLE,
		version=_L.VERSNUM,
		update=combatstats_onUpdate,
		tooltip=combatStatsIB_Tooltip,
	}
	
	registerAllEvents(self)
	
	if(CombatStatsIBFrame:IsVisible()) then
		CombatStatsIBFrameCenteredText:SetText(CS_MINIFRAME_DEFAULT_TEXT)
	end
end


local function combatStatsIB_OnClick()
	combatStatsText_OnClick()
end


function combatStatsIB_OnEnter()
	--InfoBar_ShowTooltip();
	--GameTooltip:SetOwner(this, "ANCHOR_RIGHT");
	--GameTooltip:SetText("Right-click to show stats.");
	InfoBar_ShowPluginTooltip(this);
	
	combatStatsText_OnEnter()
end


function combatStatsIB_Tooltip()
	return CS_TT_ADDON_TEXT_INFOBAR
end


--
--	Khaos Items
--

function registerKhaos()
	--msgPrint("registerKhaos",0.0, 1.0, 0.0);
	local localKhaosParseTree = {
		-- String keywords branch into other trees
		["keyword"] = {localKhaosParseTree}; -- IMPORTANT NOTE: KEYWORDS MUST BE LOWERCASE
		-- the default keyword is a special case
		default = {localKhaosParseTree}; -- Will get executed only if nothing else matches
		enable = {callback = CombatStats_ChatCommandHandler;};
		disable = {callback = CombatStats_ChatCommandHandler;};
		showonlywhentarget = {callback = CombatStats_ChatCommandHandler;};
		showalways = {callback = CombatStats_ChatCommandHandler;};
		statsonmouseover = {callback = CombatStats_ChatCommandHandler;};
		statsonclick = {callback = CombatStats_ChatCommandHandler;};
		endoffight = {callback = CombatStats_ChatCommandHandler;};
		noendoffight = {callback = CombatStats_ChatCommandHandler;};
		hide = {callback = CombatStats_ChatCommandHandler;};
		show = {callback = CombatStats_ChatCommandHandler;};
		hidettip = {callback = CombatStats_ChatCommandHandler;};
		showstats = {callback = CombatStats_ChatCommandHandler;};
		reset = {callback = CombatStats_ChatCommandHandler;};
	};
	local localKhaosSlashCommand = {
		id = "CombatStatsCommands";
		commands = {"/combatstats", "/cs"};
		parseTree = { localKhaosParseTree };
		helpText = "CombatStats functions.";
	};
	Khaos.registerOptionSet(
		"combat",
		{
			id = kAddonName;
			text = COMBATSTATS_CONFIG_HEADER;
			helptext = COMBATSTATS_CONFIG_HEADER_INFO;
			difficulty = 1;
			default = true;
			-- commands = localKhaosSlashCommand;
			options = {
				{
					id = "Header";
					type = K_HEADER;
					text = COMBATSTATS_CONFIG_HEADER;
					helptext = COMBATSTATS_CONFIG_HEADER_INFO;
					difficulty = 1;
					default = false;
				};
				{
					id = "Enable";
					type = K_TEXT;
					text = COMBATSTATS_CONFIG_ONOFF;
					helptext = COMBATSTATS_CONFIG_ONOFF_INFO;
					difficulty = 1;
					check = true;
					callback = function(state)
						if (state.checked) then
							setEnable(1);
						else
							setEnable(0);
						end
					end;
					feedback = function(state)
						if ( state.checked ) then
							return 'CombatStats is enabled.';
						else
							return 'CombatStats is disabled.';
						end
					end;
					default = {checked = true};
					disabled = {checked = false};
				};
				{
					id = "COMBATSTATS_CONFIG_USEMOUSEOVER";
					type = K_TEXT;
					check = true;
					text = COMBATSTATS_CONFIG_USEMOUSEOVER;
					helptext = COMBATSTATS_CONFIG_USEMOUSEOVER_INFO;
					difficulty = 2;
					feedback = function(state)
						if ( state.checked ) then
							return "Display on mouseover.";
						else
							return "Don't display on mouseover.";
						end
					end;
					callback = function(state)
						if (state.checked) then
							setMouseOver(1);
						else
							setMouseOver(0);
						end
					end;
					default = {checked = false};
					disabled = {checked = false};
				};
				{
					id = "COMBATSTATS_HIDEONNOTARGET_ONOFF";
					type = K_TEXT;
					check = true;
					text = COMBATSTATS_CONFIG_HIDEONNOTARGET;
					helptext = COMBATSTATS_CONFIG_HIDEONNOTARGET_INFO;
					difficulty = 2;
					feedback = function(state)
						if ( state.checked ) then
							return "Will hide on no target.";
						else
							return "Will not hide on no target.";
						end
					end;
					callback = function(state)
						if (state.checked) then
							setTarget(1);
						else
							setTarget(0);
						end
					end;
					default = {checked = false};
					disabled = {checked = false};
				};
				{
					id = "COMBATSTATS_CONFIG_ENDOFFIGHT";
					type = K_TEXT;
					check = true;
					text = COMBATSTATS_CONFIG_ENDOFFIGHT;
					helptext = COMBATSTATS_CONFIG_ENDOFFIGHT_INFO;
					difficulty = 2;
					feedback = function(state)
						if ( state.checked ) then
							return "Will display stats at end of fight.";
						else
							return "Will not display stats at end of fight.";
						end
					end;
					callback = function(state)
						if ( state.checked ) then
							setEndFight(1);
						else
							setEndFight(0);
						end
					end;
					default = {checked = false;slider = 0};
					disabled = {checked = false;slider = 0};
				};
				{
					id = "COMBATSTATS_HIDE_ONOFF";
					type = K_TEXT;
					check = true;
					text = "Hide Frame";
					helptext = "When enabled the frame over the player will be hidden";
					difficulty = 2;
					feedback = function(state)
					if ( state.checked ) then
							return "Will hide.";
						else
							return "Will not hide.";
						end
					end;
					callback = function(state)
						if (state.checked) then
							setHide(1);
						else
							setHide(0);
						end
					end;
					default = {checked = false};
					disabled = {checked = false};
				};
			};
		}
	);
	
end


-------------------------------------------------------------------------------
-- Function Prototypes - Public
-------------------------------------------------------------------------------
CombatStats_OnLoad = combatstats_onLoad
CombatStats_OnUpdate = combatstats_onUpdate
CombatStats_OnEvent = combatstats_onEvent
CombatStats_ChatCommandHandler = chatCommandHandler
CombatStatsText_OnClick = combatStatsText_OnClick
CombatStatsText_OnEnter = combatStatsText_OnEnter
CombatStats_Reset = reset
CombatStats_ResetPos = resetPos
CombatStats_Information = information
CombatStats_GetCurrentCombatData = getCurrentCombatData
CombatStats_GetFormattedCurrentCombatData = getFormattedCurrentCombatData
CombatStats_GetUpdateFreq = getUpdateFreq
--CombatStats_GetDynDPSformatStr = getDynDPSformatStr
CombatStats_GetCurrentDynDPSformatStr = getCurrentDynDPSformatStr

CombatStats_SetEnable = setEnable
CombatStats_SetTarget = setTarget
CombatStats_SetMouseOver = setMouseOver
CombatStats_SetEndFight = setEndFight
CombatStats_SetHide = setHide
CombatStats_SetShowTtip = setShowTtip
CombatStats_SetShowDPSinMini = setShowDPSinMini
CombatStats_SetShowHPSinMini = setShowHPSinMini
CombatStats_SetAutoBatReset = setAutoBatReset
CombatStats_UpdateFramePer = updateFramePer
CombatStats_SetMaxUPS = setMaxUPS

CombatStats_DumpInternalStats = dumpInternalStats
CombatStats_DumpConfigVariables = dumpConfigVariables

CombatStatsIB_OnLoad = combatStatsIB_OnLoad
CombatStatsIB_OnClick = combatStatsIB_OnClick
CombatStatsIB_OnEnter = combatStatsIB_OnEnter
--CombatStatsIB_Tooltip = combatStatsIB_Tooltip

function CombatStats_GetWoWInterface() return (_L.WoWintrfc or 0) end